/* Copyright Feb 2013 Stelios Joannou, Andreas Poyias and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef SDOM_TEST
#define SDOM_TEST

#include <stdio.h>
#include <string.h>
#include <stdexcept>

#include <time.h>

#include "../../include/SIXML/SIXML.h"

#include "Traversals.h"
#include <ctype.h>
#include <stdlib.h>
#include <unistd.h>
#include <vector>
#include <getopt.h>

#include <sys/types.h>
#include <sys/stat.h>

#include <iostream>
#include <string>
#include <sstream>
#include <queue>
#include <stdio.h>
#include <sys/resource.h>
#include <time.h>


using namespace SiXML;
using namespace std;
	int traversal=0;
	int tempTraversal=0;
	string traversalName="";
	int checkArgs=1;
	bool traverse= false;
	ofstream outFile;
	string str= "Unknown";
	string prefix="-";
	string result;
	vector<string> output;
	vector<string> outTraverse;
	vector<string> argz;
	double size;
	int repCount;
	int option;


/**
   * Returns a double as string 
   * @param double x containing the double 
   * to be converted as string
   * @return String.
   */ 
	string stringify(double x){
	  stringstream o;
	   o << x;
	   return o.str();
	}// end stringify

/**
   * Returns a double containing the file size 
   * @param string containing file name 
   * @return file size.
   */ 
	double fileSize(std::string filename){
	    FILE *p_file = NULL;
	    p_file = fopen(filename.c_str(),"rb");
	    fseek(p_file,0,SEEK_END);
	    double size = ftell(p_file);
	    fclose(p_file);
    		return size;
	} //end filesize
/**
   * Returns boolean if the file exists 
   * @param containing file name 
   * @return file size.
   */ 
	bool fexists(const char *filename){
	  ifstream ifile(filename);
	  return ifile;
	}//end fexists

/**
   * Fetches virtual memory and resident memory 
   * from "proc/self/stat" file 
   * @param double vm_usage, double resident_set 
   * @return void.
   */ 
	void process_mem_usage(double * vm_usage, double * resident_set){
			*vm_usage     = 0.0;
		*resident_set = 0.0;

		FILE * memFile = fopen("/proc/self/stat", "r");

		unsigned long vsize;
		long rss;

		char str[60];
		int tmp = 0;
		for (unsigned int i=0; i < 22; i++)
		tmp = fscanf(memFile, "%s", str);

		tmp = fscanf(memFile, "%ld", &vsize);
		tmp = fscanf(memFile, "%ld", &rss);
		*vm_usage = tmp; // just to stop warning about unused variable
		long page_size_kb = sysconf(_SC_PAGE_SIZE) / 1024; // in case x86-64 is configured to use 2MB pages
		*vm_usage     = vsize / 1024.0;
		*resident_set = rss * page_size_kb;

		fclose(memFile);
	}//end process_mem_usage
	
/**
   * Uses process_mem_usage function 
   * prints VM and physical memory currently used in KB
   * the ouput is in a csv format
   * @param no parameters
   * @return void.
   */ 
	void printMemory() {
    		stringstream sstrVM,sstrRSS;
	       double vm, rss;
	       process_mem_usage(&vm, &rss);
		sstrVM<< vm;
		sstrRSS<<rss;
	       //outFile << vm << "," << rss << ",";
		if (traverse){
		outTraverse.push_back(sstrVM.str()+","+sstrRSS.str()+",");
		}else{
		output.push_back(sstrVM.str()+","+sstrRSS.str()+",");
		}
	}//end printMemory

     int main (int argc, char **argv){

        int c;
	bool fFlag=false;
	bool rFlag=false;
	bool hFlag=false;
	bool cFlag=false;
	bool nFlag=false;
	bool failFlag=false;
	bool expFlag=false;
	int repcount=1;  
	vector<string> xmlFiles;    

	int xmlIndex;
	int countXMLs=0;
 static struct option long_options[] =
             {
               /* These options set a flag. */
               {"DOTWCOUNT", no_argument, &tempTraversal, 1},
               {"RDOTWCOUNT",   no_argument, &tempTraversal, 2},
               {"DOTWNEXTOPCOUNT", no_argument, &tempTraversal, 3},
               {"RDOTWNEXTOPCOUNT",   no_argument, &tempTraversal, 4},
               {"DOTWCOUNTFULL", no_argument, &tempTraversal, 5},
               {"RDOTWCOUNTFULL",   no_argument, &tempTraversal, 6},
               {"DOTWNEXTOPCOUNTFULL", no_argument, &tempTraversal, 7},
               {"RDOTWNEXTOPCOUNTFULL",no_argument, &tempTraversal, 8},
               {"DONODECOUNT", no_argument, &tempTraversal, 9},
               {"RDONODECOUNT",   no_argument, &tempTraversal, 10},
               {"DONODECOUNTFULL", no_argument, &tempTraversal, 11},
               {"RDONODECOUNTFULL",   no_argument, &tempTraversal, 12},
               {"ALRTW",   no_argument, &tempTraversal, 13},
               /* These options don't set a flag. 
		* Don't require arguments
		*/
               {"help",     no_argument,       0, 'h'},
               {"csv",    no_argument,         0, 'c'},
		/*These require arguments*/
               {"repcount", required_argument, 0, 'r'},
               {"name", required_argument,     0, 'n'},
               {"file",     required_argument, 0, 'f'},
               {0, 0, 0, 0}
             };


       while (1){
  
           int option_index = 0;
     	   /*set to c the next option*/
           c = getopt_long (argc, argv, "hcn:r:f:",long_options, &option_index);
     
           /*Detect the end of the options.*/
           if (c == -1)
             break;
     		
		/*switch for handling each option no matter order of input*/
           switch (c){
		/*Handling Flag options*/
             case 0:
			if (traversal==0){
				traversal=tempTraversal;
				traversalName=long_options[option_index].name;
				argz.push_back(traversalName);
				expFlag=true;
				checkArgs++;
			}else{
				cerr<<"More than one experiments entered\n One experiment at a time is required."<<endl;
			failFlag=true;
                }
               break;
             case 'h':
		if (fFlag || rFlag || cFlag ){
		hFlag=true;
		cerr<<"Cannot use -h or --help together with other options.\nHelp option usage: benchmark -h\n";
		failFlag=true;
		}else{
		hFlag=true;
               puts ("option -h\n");
		argz.push_back("-h");
		checkArgs++;
		}
               break;
             case 'r':
		rFlag=true;
		repcount=atoi(optarg);
		argz.push_back("-r");
		argz.push_back(optarg);
		checkArgs+=2;
               break;
             case 'n':
		nFlag=true;
		str=optarg;
		argz.push_back("-n");
		argz.push_back(str);
		checkArgs+=2;
               break;
	     case 'c':
		outFile.open("SiXDOM_Data.csv", std::ios::app);
		cFlag=true;
		checkArgs++;
		argz.push_back("-c");
               break;
             case 'f':
			xmlIndex=optind-1;
			checkArgs++;
			argz.push_back("-f");
		while(xmlIndex<argc){
			prefix=argv[xmlIndex];
			if(prefix.compare(0,1,"-")!=0){
				if (!fexists(prefix.data())){
					cerr<<"File '"<<prefix.data()<<"' does not exist"<<endl;
					argz.push_back("-f");
					failFlag=true;
				}
				xmlFiles.push_back(argv[xmlIndex]);
				argz.push_back(argv[xmlIndex]);
				countXMLs++;
				fFlag=true;
			}else{
				break;
			}
			xmlIndex++;
		}//end while
		if (countXMLs<1){
			cerr<<"Option -f requires at least one xml file.\nFor example -f <xmlFile1.xml> <xmlfile2.xml>.\n";
			failFlag=true;
		}
		checkArgs+=countXMLs;
               break;
             default:
               abort ();
             }// end switch
         }//end while
	
	if(!fFlag){
		if(!hFlag){
		cout<<"Type "<<argv[0]<<" followed by the Available subcomands"<<endl;
		cout<< "     "<<"-f or --file  <file1> <file2> .."<<endl;
		cout<< "     "<<"-n or --name  <experimment name>"<<endl;
		cout<< "     "<<"-c or --csv"<<endl;
		cout<< "     "<<"-r or --repcount <int>"<<endl;
		cout<< "     "<<"Possible experiments:" <<endl;
		cout<< "          "<< "--DOTWCOUNT"<<endl;
		cout<< "          "<< "--RDOTWCOUNT"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--DOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--DONODECOUNT"<<endl;
		cout<< "          "<< "--RDONODECOUNT"<<endl;
		cout<< "          "<< "--DONODECOUNTFULL"<<endl;
		cout<< "          "<< "--RDONODECOUNTFULL"<<endl;
		cout<< "          "<< "--ALRTW"<<endl;

		}
	}

	/*if the above input was wrong for any reason we abort */
	if(failFlag){
		cout<<"------------------------------------------------------"<<endl;
		cout<<"Type "<<argv[0]<<" followed by the Available subcomands"<<endl;
		cout<< "     "<<"-f or --file  <file1> <file2> .."<<endl;
		cout<< "     "<<"-n or --name  <experimment name>"<<endl;
		cout<< "     "<<"-c or --csv"<<endl;
		cout<< "     "<<"-r or --repcount <int>"<<endl;
		cout<< "     "<<"Possible experiments:" <<endl;
		cout<< "          "<< "--DOTWCOUNT"<<endl;
		cout<< "          "<< "--RDOTWCOUNT"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--DOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--DONODECOUNT"<<endl;
		cout<< "          "<< "--RDONODECOUNT"<<endl;
		cout<< "          "<< "--DONODECOUNTFULL"<<endl;
		cout<< "          "<< "--RDONODECOUNTFULL"<<endl;
		cout<< "          "<< "--ALRTW"<<endl;


	 abort();
	}

	/*checks if repcount was set without experiment option*/
	if (!expFlag && rFlag){
		failFlag=true;
		cerr<<"RepCount entered but no experiment\n";
		cout<<"Type "<<argv[0]<<" followed by the Available subcomands"<<endl;
		cout<< "     "<<"-f or --file  <file1> <file2> .."<<endl;
		cout<< "     "<<"-n or --name  <experimment name>"<<endl;
		cout<< "     "<<"-c or --csv"<<endl;
		cout<< "     "<<"-r or --repcount <int>"<<endl;
		cout<< "     "<<"Possible experiments:" <<endl;
		cout<< "          "<< "--DOTWCOUNT"<<endl;
		cout<< "          "<< "--RDOTWCOUNT"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--DOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--DONODECOUNT"<<endl;
		cout<< "          "<< "--RDONODECOUNT"<<endl;
		cout<< "          "<< "--DONODECOUNTFULL"<<endl;
		cout<< "          "<< "--RDONODECOUNTFULL"<<endl;
		cout<< "          "<< "--ALRTW"<<endl;
		abort();		
	}
	/* hFlag should be called alone*/
	if (hFlag &&  (fFlag || rFlag || cFlag)){
		cerr<<"Cannot use -h or --help together with other options.\nHelp option usage: benchmark -h\n";
		cout<<"Type "<<argv[0]<<" followed by the Available subcomands"<<endl;
		cout<< "     "<<"-f or --file  <file1> <file2> .."<<endl;
		cout<< "     "<<"-n or --name  <experimment name>"<<endl;
		cout<< "     "<<"-c or --csv"<<endl;
		cout<< "     "<<"-r or --repcount <int>"<<endl;
		cout<< "     "<<"Possible experiments:" <<endl;
		cout<< "          "<< "--DOTWCOUNT"<<endl;
		cout<< "          "<< "--RDOTWCOUNT"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--DOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--DONODECOUNT"<<endl;
		cout<< "          "<< "--RDONODECOUNT"<<endl;
		cout<< "          "<< "--DONODECOUNTFULL"<<endl;
		cout<< "          "<< "--RDONODECOUNTFULL"<<endl;
		cout<< "          "<< "--ALRTW"<<endl;
		abort();
	}else if (hFlag){
		cout<<"Type "<<argv[0]<<" followed by the Available subcomands"<<endl;
		cout<< "     "<<"-f or --file  <file1> <file2> .."<<endl;
		cout<< "     "<<"-n or --name  <experimment name>"<<endl;
		cout<< "     "<<"-c or --csv"<<endl;
		cout<< "     "<<"-r or --repcount <int>"<<endl;
		cout<< "     "<<"Possible experiments:" <<endl;
		cout<< "          "<< "--DOTWCOUNT"<<endl;
		cout<< "          "<< "--RDOTWCOUNT"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNT"<<endl;
		cout<< "          "<< "--DOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWCOUNTFULL"<<endl;
		cout<< "          "<< "--DOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--RDOTWNEXTOPCOUNTFULL"<<endl;
		cout<< "          "<< "--DONODECOUNT"<<endl;
		cout<< "          "<< "--RDONODECOUNT"<<endl;
		cout<< "          "<< "--DONODECOUNTFULL"<<endl;
		cout<< "          "<< "--RDONODECOUNTFULL"<<endl;
		cout<< "          "<< "--ALRTW"<<endl;
	}

	/* error checking
	 * if all arguments are recognised
	 */
	if (expFlag){
		option=traversal;
		argz.push_back("--"+traversalName);
	}
		
     /* Prints Arguments that are not Recognised */
      if (argz.size() < argc-1){
		int i=0; 
	for(int x=1;x<argc; x++){
		for (int y=0; y<argz.size();y++){
			if (argz[y]==argv[x])
				i++;
		}
		if (i==0){
			cerr<< "Argument not Recognised "<<argv[x]<<endl;
			abort();
		}
		i=0;	
	}
      }

///////////////////////////////end of option fetching//////////////////////////////////


	time_t start_time, finish_time;
	double elapsed_time;
	int XMLNo=xmlFiles.size();
   	SDOM_Node * docNode[xmlFiles.size()];
  	TreeWalker * myWalker2[xmlFiles.size()];

	/*if fFlag then parse the files*/
	if (fFlag){

	for (int i=0; i<xmlFiles.size();i++){

		
		start_time =time (NULL); //start time before parse
		/* print memory before parse*/
		if(cFlag){
			size=fileSize(xmlFiles[i].data());
			result=stringify(size/1024/1024);
			output.push_back(str+","+xmlFiles[i].data()+","+result+"MB,");
			printMemory();
		}
		
	  	docNode[i] = parse_file(xmlFiles[i].data());//parse file
		/*print memory after parse*/
		if(cFlag) printMemory();

	        finish_time =time (NULL);//clock();

		
		/* calculate elapsed time */
		 elapsed_time =(double)(finish_time - start_time);
		if(cFlag){ 
			result=stringify(elapsed_time)+",";
			output.push_back(result);
		}
	  	/* create treewalker for specific docnode*/
	  	myWalker2[i]=new TreeWalker(docNode[i]);
        }//end for
	}//enr fFlag if

	/*if there is an experiment then run it*/
	if (expFlag){
		repCount = repcount;
		outFile << endl;
		traverse=true;
		/*runs experiment for each file*/
		for (int i=0; i<XMLNo;i++){
		   Traversals myExperiments(myWalker2[i],docNode[i]);
		   start_time =time (NULL);
			/*print memory before traverse*/
			if(cFlag){ 
			printMemory();}	
			myExperiments.doTraversals(option,repCount);//traverse
			/* memory after traverse*/
			if(cFlag) printMemory();
		    
			finish_time =time (NULL);	
			/*calculate and print elapsed time*/ 
		    	elapsed_time =(double)(finish_time - start_time);
			if(cFlag){
				result=stringify(elapsed_time);
			  outTraverse.push_back(result);
			}
		}//end for
	}//end if expFlag

	/* Print data in csv file*/
	if(cFlag){
		int j=0;
		int k=0;
		for (int i=0; i<XMLNo;i++){
			for (j; j<(4+4*i);j++){	
				outFile << output[j].data();

			}	
			for (k; k<(3+3*i);k++){	
				outFile << outTraverse[k].data();
			}
			outFile<<endl;
		}
			outFile.close();
	}

	for (int i=0; i<XMLNo;i++){
	   delete myWalker2[i];
	   delete docNode[i];
	}
	
return 0;
}//end main
#endif
