/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou, Andreas Poyias and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */



/*

 Options when running:
 1. DOTWCOUNT,          2. RDOTWCOUNT,       3. DOTWNEXTOPCOUNT, 
 4. RDOTWNEXTOPCOUNT,    5. DOTWCOUNTFULL,      6. RDOTWCOUNTFULL,
 7. DOTWNEXTOPCOUNTFULL,8. RDOTWNEXTOPCOUNTFULL,9. DONODECOUNT, 
10. RDONODECOUNT,     11. DONODECOUNTFULL,          12. RDONODECOUNTFULL
13. ALRTW

 */
#ifndef TEST_HARNESS_H_
#define TEST_HARNESS_H_

#include <iostream>
#include "../../include/SIXDOM/SDOM_Node.h"
#include "../../include/SIXDOM/TreeWalker.h"


#include <time.h>


using namespace std;

class Traversals {
public:
  Traversals() {
  }

  Traversals(TreeWalker* walker,SDOM_Node * pnode):myNodeCount(0), myTextCount(0), myAttrCount(0), myElemCount(0),maxDepth(0),check(0),attLength(0){
    myWalker=walker;
    myNode=pnode;
    SDOM_Node* current=myWalker->getCurrentNode();
    rootNode= new  SDOM_Node(*current);
    origNode=new  SDOM_Node(*current);
    while(current!=NULL){
      current=myWalker->getLastChild();
    }
    current=myWalker->getCurrentNode();
    lastNode= new  SDOM_Node(*current);

    myWalker->setNodeNr(rootNode);
    namedNodeMap=new SDOM_NamedNodeMap(rootNode->getSucTree(),rootNode->getNodeNr(),0);
	
  }
  
  Traversals(TreeWalker* walker):myNodeCount(0), myTextCount(0), myAttrCount(0), myElemCount(0),maxDepth(0),check(0),attLength(0){
    myWalker=walker;
    SDOM_Node* current=myWalker->getCurrentNode();
    rootNode= new  SDOM_Node(*current); 
    origNode=new  SDOM_Node(*current);
    while(current!=NULL){
      current=myWalker->getLastChild();
    }
    current=myWalker->getCurrentNode();
    lastNode= new  SDOM_Node(*current);
    myWalker->setNodeNr(rootNode);
    namedNodeMap= new SDOM_NamedNodeMap(rootNode->getSucTree(),rootNode->getNodeNr(),0);
  }
  Traversals(SDOM_Node * pnode):myNodeCount(0), myTextCount(0), myAttrCount(0), myElemCount(0),maxDepth(0){
    myNode=pnode; 
    rootNode= new  SDOM_Node(*pnode); 
  }
  
  void reset(){
    myNodeCount=0; myTextCount=0; myAttrCount=0; myElemCount=0;
    myWalker->setNodeNr(rootNode);
  }
  void doTraversals(int option, int repCount);
  int matchSubString(const char * myStr, const char* subStr){
    int j=0;
    for(int i=0;myStr[i]!='\0';i++){
      if((char)myStr[i]==subStr[0])j++;    
    }
    return j;
  }
  
  void DOTWCOUNT();
  void DOTWCOUNTFULL();
  void RDOTWCOUNT();
  void RDOTWCOUNTFULL();
  void RecursiveTraversal( SDOM_Node* pnode);
  void DOTWNEXTOPCOUNT();
  void DOTWNEXTOPCOUNTFULL();
  void RDOTWNEXTOPCOUNT();
  void RDOTWNEXTOPCOUNTFULL();
  
  void DONODECOUNT();
  void RDONODECOUNT();
  void DONODECOUNTFULL();
  void RDONODECOUNTFULL();
  void ALRTW();
  void TreeWalkTest(SDOM_Node* pnode);
  
  void visitNode(SDOM_Node * current){
    if(current->getNodeType()==ELEMENT && check<2){
      myElemCount++;
      int attLength=(current->getAttributes())->getLength();
      myAttrCount+=attLength;
      
      for(int i=0;i<attLength;i++){
	 current= current->getAttributes()->item(i);
      }
    }else{ 
      if(current->getNodeType()==TEXT){ 
	myTextCount++;
	countMatch+=matchSubString(current->getNodeValue(), "x");
      }
    }
  }
  
  int reversePath(SDOM_Node * current1){
    origNode->setNode(myWalker->getCurrentNode());
    int i=0;
    while(current1!=NULL){
      current1=myWalker->getParentNode();
      i++;
    }
    myWalker->setNodeNr(origNode);
    return i-1;
  }
  
 
  int getNodeCount(){return myNodeCount;}
  int getElemCount(){return myElemCount;}
  int getTextCount(){return myTextCount;}
  int getAttrCount(){return myAttrCount;}

private:
  int myNodeCount, myTextCount, myAttrCount, myElemCount,maxDepth;
  int countMatch;
  int check;
  int attLength;
  SDOM_Node * myNode;   
  SDOM_Node * origNode;  
  SDOM_Node * rootNode;
  SDOM_Node * lastNode;
  TreeWalker* myWalker;
  SDOM_NamedNodeMap *namedNodeMap;
};
#endif
