/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */


#ifndef ATTRIBUTEDS_H_
#define ATTRIBUTEDS_H_

#include <vector>
#include <math.h>
#include "NamePool.h"
#include "../PrefixSumDS/BitString.h"
#include "myUtils.h"
#include "IdHashMap.h"
#if defined(TEXTDS)
#include "TextDS.h"
#else
#include "TextBZDS.h"
#endif

using namespace std;
class TextDS;
//class SuccinctDS;

class AttributeDS{
 public:

  //default constructor
  AttributeDS()throw(bad_alloc){
    maxValue=0;
    idCode1=0;
    idCode2=0;
    myIdMap = new IdHashMap<>(0);
    namecodeVector=new MyVector<unsigned int>(1000);
    nsIdVector=new MyVector<unsigned int>(100);
    textDS=NULL;
  }
  AttributeDS(unsigned char * numbers, int length)throw(bad_alloc);
  AttributeDS(const AttributeDS& x)throw(bad_alloc){
    maxLength=x.maxLength;
    nodeCount=x.nodeCount;
    maxValue=x.maxValue;
    bv_length=x.bv_length;
    idCode1=x.idCode1;
    idCode2=x.idCode2;
    parentRank=x.parentRank;
    attrCount=x.attrCount;
    attPosition=x.attPosition;
    nullNode=x.nullNode;
    bs_namecodes1=x.bs_namecodes1;
    namepool=x.namepool;
    run1FID=x.run1FID;
    run0FID=x.run0FID;
  }

  ~AttributeDS()throw(){
    delete bs_namecodes1;
    delete myIdMap;
	if (run1FID != NULL)
		delete run1FID;
	delete run0FID;
  }
  void setup(unsigned char * numbers, int length);
  
  //starting from zero;
  inline int getBit(int i, const unsigned char*eBytes, int length){
    int mask=7; int j=i;
    if(i<length){  
      i=i&mask;
      if(j==0)j++;
      return (int)(eBytes[j>>3]>>(8-(i+1)))&1;
    }
    return -1;
  }
  
  inline void addArray(int &size,unsigned char*array, int &z, unsigned char &temp){
    if(size==8){
      array[z]=temp;
      size=0;temp=0;
      z=z+1;
    }
  }
  
  //attribute operations
  //counting from 1
  int attParent(int attNode);
  
  inline int attNodeCount(int parent){
    if(!run0FID->getInternal(parent-1))
      return 0;
    parentRank=run0FID->rank1(parent);  
    int num2a=0;
    if(parentRank!=1)
      num2a = run1FID->select1(parentRank-1);
    int num3a = run1FID->select1(parentRank)-num2a;
    attPosition = num2a+1;
    return num3a;
  }
  
  inline int getAttNode(int parent, int i){
    //need to discuss if we need these default null cases.
    if(!run0FID->getInternal(parent-1))return -1;
    int num1a=run0FID->rank1(parent);
    int num2a=0;
    if(num1a!=1)
      num2a= run1FID->select1(num1a-1);
    int num3a=run1FID->select1(num1a);
    //cout<<" i:"<<i<<" num1a:"<<num1a<<", num2a:"<<num2a<<", num3a:"<<num3a<<endl;
    if((num3a-num2a)<i)
      return -1;
    return num2a+i;
  }

  //int attNodeCount(int parent);
  
  void setIDsCode(int nc){
    if(idCode1==0)
      idCode1=nc;
    else
      idCode2=nc;
  }
  
  bool checkIDCode(int nc){
    return (nc==idCode1 || nc==idCode2);
  }
  
  int getIDShortcode(){
    if(idCode1==0)
      return idCode2;
    else
      return idCode1;
  }
  
  inline int attTotalNodeCount(){
    return nodeCount;
  }
  
  inline bool hasAttributes(int parent){
    return run0FID->getInternal(parent-1)==1;
  }
  
   
  int previousSibling(int att);
  
  int nextSibling(int att);
  
  inline void setAttrCount(int i){attrCount=i;}
  inline int getAttrCount(){return attrCount;}
  
  //general operations
  
  inline const char* getQName(unsigned int attNode){
    
    if((getShortCode(attNode))<13)return ""; 
    return namepool->getQName(getShortCode(attNode)-13);  
  }
   
  inline char* getPrefixName(unsigned int attnode){
    return namepool->getPrefixName(getShortCode(attnode)-13);
  }
  
  const char* getURI(unsigned int attnode){
    return namepool->getURI(getShortCode(attnode)-13);  
  
  }
  
  unsigned int getURICode(unsigned int attnode){
    int ncode = namepool->getURICode(getShortCode(attnode)-13);
    return ncode;
  }
  
  inline const char* getLocalName(unsigned int attnode){
    return namepool->getLocalName(getShortCode(attnode)-13);
  }
  
  unsigned int getShortCode(char* prefix, char* uri, char* localName);
  
  inline unsigned int getShortCode(const unsigned int &attnode){
    int number=(attnode)*(maxLength);
    return bs_namecodes1->subString(number+1,number+(maxLength));
  }
  
  unsigned int encode(char* prefix, char* uri, int uriId, char* localName);
  
  void addShortCode(unsigned int code){

    if(maxValue<code)maxValue=code;
    namecodeVector->push_back(code);
  }
  unsigned int alloc_Namepool(char* prefix, char* uri, int uriId, char* localName);
  
  inline unsigned int encode(const unsigned int &type){
    if(maxValue<type)maxValue=type;
  
    namecodeVector->push_back(type);
    
  }
  
  //get full namecode
  inline unsigned int decode(unsigned nc){
    return namepool->decode(nc);
  }
  
  inline NamePool &getNamePool(){return *namepool;} 
  inline void setNamePool(NamePool* nPool){
    namepool=nPool;
  } 
  
  inline unsigned int getOtherNodeCount(){return nodeCount;}

  void addBaseURI(int scode){
    baseURICode=scode;
  }

  int getBaseURICode(){
    return baseURICode;
  }
    
  void compactNameCodes(){
    nodeCount=namecodeVector->size();
    unsigned int *lengths= new unsigned int[namecodeVector->size()];
    maxLength= (char)ceil(logBase2(maxValue));

    for(int i=0;i<namecodeVector->size();i++){
      lengths[i]=maxLength;
    }
   
    bs_namecodes1= new BitString(namecodeVector->getData2(), lengths, nodeCount);
    
    delete []lengths;
    delete namecodeVector;
    delete nsIdVector;
  }
  
  inline double logBase2(const double &x){
    return (double)((log (x))/(log (2.0)));
  }
  
  //space usage info
  
  double spaceUsage(){
   
    return (spaceusage/8)+bs_namecodes1->spaceUsage();
  }
  //
  inline int getAttPosition(){return attPosition;}
  

  bool addNewID(const char *s, int lengthStr, int s_index,int n_2n){
    return myIdMap->incword(s, lengthStr, s_index, n_2n);
  }
  int findIDElem(const char *s){
    if (textDS == NULL) 
	return 0;
    unsigned int hcode = myIdMap->hash(s);
    MyVector<node_id> * p =  myIdMap->getNode(hcode);
    if (p == NULL)
	return 0;

    for (unsigned int i=0; i<p->size(); i++){
      

      if(strcmp(textDS->getString(p->at(i).str_index),s) == 0) {
	return p->at(i).node_2n;
      }
    }
    return 0;

    
  }

  void setTextDS(TextDS * tDS){
    textDS=tDS;
	
  }
 private:
  int spaceusage;
  char maxLength;
  int nodeCount;
  unsigned int maxValue;
  int bv_length;
  short idCode1;
  short idCode2;
  short baseURICode;
  short namespaceCode;
  int parentRank;
  int attrCount;//not needed we have otherNodeCount
  int  attPosition; //needed for NamedNodeMap
  nodeValue nullNode; //this is not used anymore
  MyVector<unsigned int>*namecodeVector;
  MyVector<unsigned int>*nsIdVector;
  BitString *bs_namecodes1;
  NamePool *namepool;
  tFID * run1FID;
  tFID * run0FID;
  IdHashMap <>*myIdMap;
  TextDS * textDS;
};

	
#endif
