/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef DOCTYPE_H_
#define DOCTYPE_H_

#include "NamePool.h"
#include "STree/myUtilsRoot.h"


using namespace std;

typedef struct{
	int name;
	short publicId;
	short systemId;
}DeclaredNode;

typedef struct{
	DeclaredNode myEntityNode;
	nodeValue entityRef; 
}EntityNode;

typedef struct{
	DeclaredNode myNotationNode;
}NotationNode;

class DocType{
 public:

	//default constructor
	DocType(const char* n,int l1,const char* pubId,int l2,const char* sysId, int l3,NamePool* namepool)throw(bad_alloc);
  
	DocType(int i_)throw(bad_alloc);
	DocType(const DocType& x)throw(bad_alloc);
	~DocType()throw(){}
 
	void addEntity(const char* name,int length,const char* pubId,const char* sysID);
	void addEntityReference(unsigned int scode, nodeValue i);
	//find entity
	//then add the nodevalue to its vector

	void addNotation(const char* name,int length,const char* pubId,const char* sysID);
	void setEntityRef(nodeValue i);


	inline EntityNode getEntity(int index){
		if(index<en_count)
			return myEntity[index];
		return nullEntity;
	}

	inline NotationNode getNotation(int index){
		if(index<not_count)
			return myNotation[index];
		return nullNotation;
	}

	//returns the namecode
	inline int getEntityByIndex(int index){return getEntity(index).myEntityNode.name;}
	//returns the namecode
	inline  int getNotationByIndex(int index){return getNotation(index).myNotationNode.name;}

	EntityNode findEntity(int nc);
	int findEntityIndex(int nc);
	NotationNode findNotation(int nc);

	inline const char* getName(int ncode){return namepool->getName(ncode-13);} 
	inline unsigned int getNameCode(const char*name){return namepool->findWord(name,0);}  //returns namecode
	inline unsigned int getShortCode(const char*name){return namepool->findWordIndex(name,0)+13;}  //returns namecode
	inline const char* getPublicID(DeclaredNode n){return IdVector->at(n.publicId);}
	inline const char* getSystemID(DeclaredNode n){return IdVector->at(n.systemId);}
	inline const char* getDocTypeName(){return namepool->getName(doctypeNode.name-13);} 
	inline const char* getDocTypePublicID(){return IdVector->at(doctypeNode.publicId);}  
	inline const char* getDocTypeSystemID(){return IdVector->at(doctypeNode.systemId);}


	inline int getEntityCount(){return en_count;}
	inline int getNotationCount(){return not_count;}

	//space usage info
	double spaceUsage(){
		return spaceusage;
	}
  
	void finalize();
  
	void testCode(){
		for(int i=0;i<en_count;i++){
			cout<<myEntity[i].myEntityNode.name<<", ";
		}cout<<endl<<endl;;

	}
  
  
 private:
	int spaceusage;
	MyVector<EntityNode>*EntityVect;
	MyVector<NotationNode>*notationVect;
	MyVector<const char*>*IdVector;//public and system IDs
	NamePool* namepool;
	EntityNode* myEntity;
	NotationNode* myNotation;
	//nodeValue*entityRef;
	DeclaredNode nullNode;
	EntityNode nullEntity;
	NotationNode nullNotation;
	short en_count;
	short not_count;
	DeclaredNode doctypeNode;
	int IDCount;
};
#endif
