/*
 * editted in C++ by O'Neil Delpratt
 * of:  (original copyright follows)
 */

/* Copyright (C) 1999 Lucent Technologies */
/* From 'Programming Pearls' by Jon Bentley */

//September 2006


#ifndef HASHMAP_H_
#define HASHMAP_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>
#include <iostream>
#include "MyVector.h"
#include <math.h>


using namespace std;

struct node_t {
  int nc_index;//namecode index
  int str_index;
  int urlId; //could make this a short int
  node_t *next;
};

template <typename T>
struct alist_t {
  alist_t(size_t elms) : next(0) {
    area = new T[elms];
  }
  ~alist_t() {
    if (next) delete next;
    delete[] area;
  }
  T* allocate(size_t elms) {
    alist_t *n = new alist_t<T>(elms);
    n->next = next;
    next = n;
    return n->area;
  }
  T *area;
  alist_t<T> *next;
};

/*template <unsigned NHASH = 1023,    unsigned MULT = 31,
  unsigned NODEGROUP = 1023, unsigned CHARGROUP = 1000>*/
template <unsigned NHASH = 1023,    unsigned MULT = 31,
          unsigned NODEGROUP = 1, unsigned CHARGROUP = 1>
class HashMap {
public:
  HashMap(int s)
    : bin(NHASH,0), nodesleft(0), charsleft(0),
      nglist(NODEGROUP), cglist(CHARGROUP), free_index(0),items_added(0),
      strCap(100),strNext(0),strLenStats(0)
  {
    
    str_dataV=new MyVector<char*>(200);
  }

  ~HashMap(){
	for (unsigned int i=0; i < str_dataV->size(); i++)
		delete [] str_dataV->at(i);
	delete str_dataV;
    bin.clear();
    
  }

int getStrLength(const char *s)const {
  int len=0; 
  while(s[len]!='\0')
    len++;
  return len;
 } 
  


 unsigned hash(const char *p, int n) const
  {
  unsigned int h = 0;
int test23;
n--;
  for(int i=0;i<n;i++){
      h+=pow(31,n-1-i) *(int)p[i];
  }	
    return (h & 0x7fffffff) % 1023;
  }

  unsigned hash(const char *p) const
  {
    int n =getStrLength(p);
    unsigned int h = 0;
    for(int i=0;i<n;i++){
      h+=pow(31,n-1-i) *p[i];
    }	
    return (h & 0x7fffffff) % 1023;
  }

  unsigned int incword(const char *s, int lengthStr, int id,int new_ncIndex)
  {
    node_t* p;
    int h = hash(s, lengthStr);
    int depth=1;
    unsigned int fingerprint=0;
    int temp_index=0;
    bool sameIndex=false; //used for same localnames, diff uri
    for (p = bin[h]; p; p = p->next){
      if (strcmp(s,str_dataV->at(p->str_index)) == 0) {
	if(id==p->urlId){
	  fingerprint=(depth<<10)|h;
	  nameCode_index=p->nc_index;
	  return fingerprint;
        }
	else{
	  sameIndex=true;
	  temp_index =p->str_index;
	  
	}
      } 
     depth++;
    }
    p = nmalloc();
    if(!sameIndex){
      
      str_new=new char[lengthStr];
      strLenStats+=lengthStr;
      for(int j=0;j<lengthStr;j++){
	str_new[j]=s[j];
      }
	      
      p->str_index = free_index;
      free_index++;
      items_added++;
    
      str_dataV->push_back(str_new);
      strNext++;
    }else
      p->str_index = temp_index;  
    sameIndex=false;
  
    p->urlId=id;
    p->nc_index=new_ncIndex;
    nameCode_index=new_ncIndex;
    
    //node at code does not exist
    if(bin[h]==NULL){
      p->next = NULL;//bin[h];
      bin[h] = p;
    }
    //node(s) at code exist - add to the next*
    else{
      for (node_t *e = bin[h]; e; e = e->next){
	if(e->next==NULL){
	  e->next=p;
	  p->next=NULL;
	  break;
	}
      }
    }
    fingerprint=(depth<<10)|h;

    return fingerprint;
  }
  
 
  unsigned int findWord(const char *s, int id)
{
  node_t* p;
 
  int h = hash(s);
  int depth=1;
  
  for (p = bin[h]; p; p = p->next){
    if (strcmp(s, str_dataV->at(p->str_index)/*str_data[p->str_index]*/) == 0) {
	if(id==p->urlId){
	  return (depth<<10)|h;
        }
      } 
      depth++;
    }
   return 0; 
  }

unsigned int getWordCode(const char *s){
    node_t* p;
    int h = hash(s);
    int depth=1;

    for (p = bin[h]; p; p = p->next){
      if (strcmp(s,str_dataV->at(p->str_index)/* str_data[p->str_index]*/) == 0) {
	  return (depth<<10)|h;
      } 
      depth++;
    }
   return 0; 
  }
      
  unsigned int findWordIndex(const char *s, int id)
  {
    node_t* p;
    int h = hash(s);
    int depth=1;
    for (p = bin[h]; p; p = p->next){
      if (strcmp(s,str_dataV->at(p->str_index)/* str_data[p->str_index]*/) == 0) {
	if(id==p->urlId){
	  return p->nc_index;
        }
      } 
      depth++;
    }
   return -1; 
  }
  
  
  node_t *const * begin() const { return &bin[0]; }
  node_t *const * end() const { return &bin[NHASH]; }
  
  node_t *const *getNode(int code)const{
    if(code<NHASH)
      return &bin[code];  
    return NULL;
  }
  
  const char* getData(int i){
    return str_dataV->at(i)/*str_data[i]*/;
  }
   
  int getNameCodeIndex(){
    return nameCode_index;
  }
  
  //space in Bytes
  int spaceUsage(){
  
    return (items_added*16+1023*4+strCap*4+strCap+strLenStats*8);
  }

private:
  std::vector<node_t*> bin;
  int items_added;
  int nodesleft;
  int nameCode_index;
  node_t* freenode;
  int free_index;
  int charsleft;
  char *freechar;
  int strLen, strLenStats /*used for stats purpose*/;
  alist_t<node_t> nglist;
  alist_t<char> cglist;

  int strCap, strNext;
  char* str_new;
  char** str_data;
  MyVector<char*>*str_dataV;
  char **str_dataTemp;

  node_t* nmalloc()
  {
    if (nodesleft == 0) {
      freenode = nglist.allocate(NODEGROUP);
      nodesleft = NODEGROUP;
    }
    nodesleft--;
    return freenode++;
  }

  char *smalloc(int n)
  {
    if (charsleft < n) {
      freechar = cglist.allocate(n+CHARGROUP);
      charsleft = n+CHARGROUP;
    }
    charsleft -= n;
    freechar += n;
    return freechar - n;
  }
};
#endif
