/*
 * editted in C++ by O'Neil Delpratt
 * of:  (original copyright follows)
 */

/* Copyright (C) 1999 Lucent Technologies */
/* From 'Programming Pearls' by Jon Bentley */

//September 2006


#ifndef IDHASHMAP_H_
#define IDHASHMAP_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <vector>
#include <iostream>
#include "MyVector.h"
#include <math.h>


using namespace std;

struct node_id {
  int str_index;
  int node_2n; //could make this a short int
};

template <typename T>
struct alist_id {
  alist_id(size_t elms) : next(0) {
    area = new T[elms];
  }
  ~alist_id() {
    if (next) delete next;
    delete[] area;
  }
  T* allocate(size_t elms) {
    alist_id *n = new alist_id<T>(elms);
    n->next = next;
    next = n;
    return n->area;
  }
  T *area;
  alist_id<T> *next;
};

/*template <unsigned NHASH = 1023,    unsigned MULT = 31,
  unsigned NODEGROUP = 1023, unsigned CHARGROUP = 1000>*/
template <unsigned NHASH = 1023,    unsigned MULT = 31,
          unsigned NODEGROUP = 1, unsigned CHARGROUP = 1>
class IdHashMap {
public:
  IdHashMap(int s)
    : bin(NHASH,0), nodesleft(0), charsleft(0),
      nglist(NODEGROUP), cglist(CHARGROUP), free_index(0),items_added(0),
      strCap(100),strNext(0),strLenStats(0)
  {
  }

  ~IdHashMap(){
	for (unsigned int i=0; i<bin.size(); i++) {
		if (bin.at(i) != NULL)
			delete bin.at(i);
	}
    bin.clear();

  }

int getStrLength(const char *s)const {
  int len=0; 
  while(s[len]!='\0')
    len++;
  return len;
 } 
  
 unsigned hash(const char *p, int n) const
  {
  unsigned int h = 0;
  n--;
  for(int i=0;i<n;i++){
      h+=pow(31,n-1-i) *(int)p[i];
  }	
   // return (h & 0x7fffffff) % 388284;
	return (h & 0x7fffffff) % 1023;
  }

  unsigned hash(const char *p) const
  {
    int n =getStrLength(p);
    unsigned int h = 0;
    for(int i=0;i<n;i++){
      h+=pow(31,n-1-i) *p[i];
    }	
//    return (h & 0x7fffffff) % 388284;
	return (h & 0x7fffffff) % 1023;
  }

  bool incword(const char *s, int lengthStr, int s_index,int n_2n)
  {
    node_id p;
    int h = hash(s, lengthStr);
    int depth=1;
    unsigned int fingerprint=0;
    int temp_index=0;
	      
    p.str_index = s_index;
    items_added++;
    strNext++;
  
    p.node_2n=n_2n;

    //node at code does not exist
    if(bin[h]==NULL){
	bin[h] = new MyVector<node_id>();
	bin[h]->push_back(p);
    }
    //node(s) at code exist - add to the next*
    else{
	// by stelios
	bin[h]->push_back(p);
    }
    return true;
  }
  
 
  unsigned int findWord(const char *s, int id)
{
  node_id* p;
 
  int h = hash(s);
  int depth=1;
   return 0; 
  }

unsigned int getWordCode(const char *s){
    node_id* p;
    int h = hash(s);
    int depth=1;
   return 0; 
  }
      
  unsigned int findWordIndex(const char *s, int id)
  {
    node_id* p;
    int h = hash(s);
    int depth=1;
   return -1; 
  }
  
  
  node_id *const * begin() const { return &bin[0]; }
  node_id *const * end() const { return &bin[NHASH]; }
  
  MyVector<node_id> * getNode(int code){
    if(code<NHASH)
      return bin[code];  
    return NULL;
  }
  
  const char* getData(int i){
    return NULL;//str_dataV->at(i);
  }
   
  int getNameCodeIndex(){
    return nameCode_index;
  }
  
  //space in Bytes
  int spaceUsage(){
    return (items_added*16+1023*4+strCap*4+strCap+strLenStats*8);
  }

private:
  std::vector<MyVector<node_id> *> bin;
  int items_added;
  int nodesleft;
  int nameCode_index;
  node_id* freenode;
  int free_index;
  int charsleft;
  char *freechar;
  int strLen, strLenStats /*used for stats purpose*/;
  alist_id<node_id> nglist;
  alist_id<char> cglist;
  int strCap, strNext;
  char* str_new;
  char** str_data;
  char **str_dataTemp;
};
#endif
