/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */


#ifndef NTEXTDS_H
#define NTEXTDS_H

#define TBSIZE 64
#define TBPOW 6

#include <vector>

using namespace std;

struct TextBlock {
	unsigned long * loff;
	char * text;
	unsigned short soff[TBSIZE];
};


class NTextDS {
public:
	NTextDS() {
		blockCapacity = 1;
		blockSize = 0;				
		size = 0;		
		indexBlock = new TextBlock*[1];
	}
	
	~NTextDS() {
		for (unsigned int i=0; i<blockSize; i++) {
			if (indexBlock[i]->loff != NULL)
				delete [] indexBlock[i]->loff;
			if (indexBlock[i]->text != NULL)
				delete [] indexBlock[i]->text;
			delete indexBlock[i];
		}
		delete [] indexBlock;

	}

	void finalizeDS() {}

	void addBlock(vector<char *> * strings, unsigned int charsno, TextBlock * t) {
		
		char * chars = new char[charsno];
		unsigned int charsused = 0;
		size += strings->size();

		for (unsigned int i=0; i<strings->size(); i++)	{
			strcpy(&chars[charsused], strings->at(i));
			charsused += ((t->soff[i]));
			delete [] strings->at(i);
		}
		strings->clear();

		t->text = chars;
		
		if (blockSize == blockCapacity) {
				unsigned int newSize = (blockCapacity << 1);
				TextBlock ** newIndex = new TextBlock*[newSize];
				for (unsigned int i=0; i<blockSize; i++)
						newIndex[i] = indexBlock[i];	
				delete [] indexBlock;
				indexBlock = newIndex;
				blockCapacity = newSize;
		}
		indexBlock[blockSize] = t;
		blockSize ++;
	}
		
	char * getString(unsigned int index) {
		unsigned int bIndex = index >> TBPOW;							
		unsigned int oIndex = index & ((1<<TBPOW)-1);

		TextBlock * current = indexBlock[bIndex];
		unsigned long int currentOffset = 0;
		if (current->loff == NULL) {
			for (unsigned int i=0; i<oIndex; i++) {
					currentOffset += current->soff[i];
			}
		} else {
			for (unsigned int i=0; i<oIndex; i++) {
					currentOffset += current->loff[i];
			}
		}
	
		return &current->text[currentOffset];
	}

	unsigned int getSize() {
		return size;
	}

private:
	TextBlock ** indexBlock;
	unsigned int blockSize;
	unsigned int blockCapacity;
	unsigned int size;
};

#endif
