/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef NAMECODEDS_H_
#define NAMECODEDS_H_

#include <vector>
#include <math.h>
#include "NamePool.h"
#include "../PrefixSumDS/BitString.h"

using namespace std;

class NameCodeDS{
 public:

	//default constructor
	NameCodeDS()throw(bad_alloc);
	NameCodeDS(int i_)throw(bad_alloc);
	NameCodeDS(const NameCodeDS& nCodeDS)throw(bad_alloc){
		maxLength=nCodeDS.maxLength;
		nodeCount=nCodeDS.nodeCount;
		maxValue =nCodeDS.maxValue;
		bs_namecodes=nCodeDS.bs_namecodes;
		namepool=nCodeDS.namepool;
	}
  
	~NameCodeDS()throw(){
		delete bs_namecodes;
		delete namepool;
	}

	inline short getNodeType(unsigned int other){
		//    if(other<0)return 0;
		int number=(other)*(maxLength);
		short scode= bs_namecodes->subString(number+1,number+(maxLength));
		if(scode>12)
			return 1;
		else return scode;
	}
  
	// const char* getQName(unsigned int othernode);
	const char* getMinorityNode(int scode);
     
	inline const char* getPrefixName(unsigned int othernode){
		int scode= getShortCode(othernode);
		if(scode<13)
			return getMinorityNode(scode);
		return namepool->getPrefixName(scode-13);
	}
  
	bool checkUriId(int id){
		return namepool->checkUriId(id);
	}
  
	const char* getURI(unsigned int othernode);
    
	unsigned int getURICode(unsigned int othernode);
  
	//  const char* getLocalName(unsigned int othernode);

  
	inline const char* getLocalName(unsigned int othernode){
    
		int scode= getShortCode(othernode);
		if(scode<13)
			return getMinorityNode(scode);
		return namepool->getLocalName(scode-13);
    
	}
  
	inline const char* getQName(unsigned int othernode){

		int scode= getShortCode(othernode);
		//cout<<"scode:"<<scode<<endl;
		if(scode<13)
			return getMinorityNode(scode);
		//cout<<"scode:"<<scode;return "";
		return namepool->getQName(scode-13); 
	}
  
	unsigned int getShortCode(char* prefix, char* uri, char* localName){
		return namepool->findWord1(localName,uri);
	}
  
	inline unsigned int getShortCode(const unsigned int &othernode){
		int number=(othernode)*(maxLength);
		return bs_namecodes->subString(number+1,number+(maxLength));
	}
  
	unsigned int encode(int urlId,const char* newPrefix,int lengthPrefix,
			    const char* newUrlName,int lengthURI,const char* newname,int lengthName);
  
	unsigned int encodeLO(const unsigned int &type,int uriId,const char* prefix,int lengthPrefix,
			      const char* uri,int lengthURI,const char* localName,int lengthName);
  
	unsigned int alloc_Namepool(int uriId, const char* prefix, int lengthPrefix,const char* uri, 
				    int lengthURI, const char* localName,int lengthName);
 
	unsigned int alloc_Namepool(int uriId, MyVector<const char>* prefix, int lengthPrefix,MyVector<const char> * uri,
				    int lengthURI, MyVector<const char>* localName,int lengthName);
 
	inline unsigned int encode(const unsigned int &data){
		if(maxValue<data)maxValue=data;
		namecodeVector->push_back(data);
		nodeCount++;
	}
	
	int getMaxValue(){
		return maxValue;
	}
  
	unsigned int decode(unsigned nc);//get full namecode
  
	inline NamePool *getNamePool(){return namepool;} //check that we are returning a pointer to object and not a copy
  
	inline unsigned int getOtherNodeCount(){return nodeCount;}
	//unsigned int encodeNS(char* prefix, char* uri, int uriId);
  
	void compactNameCodes();
  
	inline double logBase2(const double &x){
		return (double)((log (x))/(log (2.0)));
	}
  
	inline void vectorCopy(vector<unsigned int>&myVector){
		for(int i=0;i<myVector.size();i++){
			namecodeVector->push_back(myVector[i]);
		}
	}
  
	inline void addShortCode(unsigned int sCode){
		namecodeVector->push_back(sCode);
		nodeCount++;
	}
  
	//space usage info
	double spaceUsage(){
		double num=bs_namecodes->spaceUsage()+namepool->spaceUsage(); 
   
		return num;
	}
	//

 private:

	char maxLength;
	int nodeCount;
	unsigned int maxValue;

	MyVector<unsigned int>*namecodeVector;
	MyVector<unsigned int>*nsIdVector;
	BitString *bs_namecodes;
	NamePool *namepool;
};
#endif
