/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */


#ifndef NAMEPOOL_H_
#define NAMEPOOL_H_

#include "STree/myUtilsRoot.h"


#include "HashMap.h"
#include "MyVector.h"


using namespace std;

class NamePool{
 public:

	//default constructor
	NamePool()throw(bad_alloc);
	NamePool(int i_)throw(bad_alloc);
	NamePool(const NamePool& x)throw(bad_alloc);
	~NamePool()throw(){
		for(int i=0;i<uriCap;i++){
			for(int j=0;j<prefixesSize;j++)
				prefixForUri[i][j].erase(0,prefixForUri[i][j].length());
			delete [] prefixForUri[i];
      
			if (uriNames[i] != NULL)
				delete [] uriNames[i];
		}

		for (unsigned int i=0; i<QNamesSize; i++) {
			if (QNames[i] != NULL)
				delete [] QNames[i];
		}
		delete [] QNames;

		delete [] prefixForUri;
		delete [] uriNames;
		delete [] nameCodeA;
		delete myMap;
	}
  
	void cleanUp(){
		if(uriSize==0){
			for(int i=0;i<uriCap;i++){
				for(int j=0;j<prefixesSize;j++)
					prefixForUri[i][j].erase(0,prefixForUri[i][j].length());
				
			}
			delete [] prefixForUri;
			delete [] uriNames;
		}
	}

	//  const char* getQName(unsigned int shortCode);
  
	inline const char* getQName(unsigned int shortCode){
		unsigned int ncode=decode(shortCode);
		
		int myUriIndex=getURICode2(ncode);
		
		if(myUriIndex!=1 && myUriIndex!=4 && myUriIndex!=0){
			return QNames[shortCode];
		}
		return getName2(ncode);
	}

	
	inline unsigned int getURICode2(unsigned int ncode){
		int depth =(ncode>>10)&0x3ff;
		node_t*const*p=  myMap->getNode(ncode&0x3ff);
		const node_t *e = *p;
		int num=1;
		for (const node_t *e1 = *p; e1; e1 = e1->next,num++){
			if(num>=depth){
				return e1->urlId;
			}
		}
		return e->urlId;
	}

  
	inline unsigned int getHashCode(const char * s){
		return myMap->hash(s);
	}
  
	inline unsigned int findWord(const char *s, int id){return myMap->findWord(s,id);}
  
	inline unsigned int findWord1(const char * uri, const char *s){
		int id=0;
		if(uri[0]!='\0'){
			std::cout << "here" << std::endl;
			//std::cout << "uriSize: " << uriSize << std::endl;
			for(int i=1;i<uriSize;i++){
				if (strcmp(uri, uriNames[i]) == 0) {
					id=i;
					return myMap->findWordIndex(s,id);
				}
			}
		}
		return  myMap->findWordIndex(s,0);
	}

	inline unsigned int findWordIndex(const char *s, int id){return myMap->findWordIndex(s,id);}
  
	//repeated  code here -  think about having one code
	//given fingerprint
	const char* getName(unsigned int nc){
		nc=decode(nc);
		int depth =(nc>>10)&0x3ff;
		node_t*const*p=  myMap->getNode(nc&0x3ff);
        
		const node_t *e = *p;
		if(e==NULL){return '\0';}
		int num=1;
		for (const node_t *e1 = *p; e1; e1 = e1->next,num++){
			if(num>=depth){
				return myMap->getData(e1->str_index); 
			}
		}
		return myMap->getData(e->str_index); 
		
	}
  
	//long nc
	const char* getName2(unsigned int nc){
   
		int depth =((nc>>10)&0x3ff);
		node_t*const*p=  myMap->getNode(nc&0x3ff);
        
		const node_t *e = *p;
		if(e==NULL){return '\0';}
		int num=1;
		for (const node_t *e1 = *p; e1; e1 = e1->next,num++){
			if(num>=depth){ 
				
				return myMap->getData(e1->str_index); 
			}
		}
		return myMap->getData(e->str_index); 
		
	}
  
	  
	inline char* getPrefixName(const unsigned int &shortCode){
		unsigned int ncode=decode(shortCode); 
		int uriIndex=getURICode(shortCode);
		int prefixIndex=(ncode>>20)&0x3ff;
		string temp=" ";
		if(uriIndex==0 && prefixIndex==0)return (char*)temp.c_str();
        
		return (char*)prefixForUri[uriIndex][prefixIndex].c_str();
	}
  
  
	inline char* getURI(const unsigned int & shortCode){
		return uriNames[getURICode(shortCode)];
	}
  
  
	unsigned int getURICode(unsigned int shortcode);
  
	int getUriId(const char* uri);
  
	inline bool checkUriId(int id){
		return id<=uriSize;
	}
  
	unsigned int getPrefixId(char* prefix){
		return 0;
	}
  
	inline const char* getLocalName(unsigned int shortCode){
		//if code exists
		if(shortCode>=ncCap)return NULL;
    
		int ncode =decode(shortCode);
		int depth =(ncode>>10)&0x3ff;
		node_t*const*p=  (myMap->getNode(ncode&0x3ff));
		const node_t *e = *p;
		int num=0;
		for (const node_t *e1 = *p; e1; e1 = e1->next,num++){
			if(num>=depth){
				return myMap->getData(e1->str_index); 
			}
		}
		return myMap->getData(e->str_index); 
	}
  




	unsigned int getShortCode(char* prefix, char* uri, char* localName);
  
	inline unsigned int decode(const int &shortCode){
		return nameCodeA[shortCode];
	}//get full namecode
  
	MyVector<const char*>* getAllQNames(){
		MyVector<const char*>  *QNamesVect = new MyVector<const char*>(100);
		int num=0;
		int depth =0;
		int ncode=0;
		node_t*const*p;
		const node_t *e;char * myURI;
		for(int i=0;i<ncSize;i++){
			ncode =nameCodeA[i]&0x3ff;
			depth =(ncode>>10)&0x3ff;
			p=  (myMap->getNode(ncode));
			e = *p;
			num=0;
			for (const node_t *e1 = *p; e1; e1 = e1->next,num++){
				if(num>=depth){
					myURI = getURI(i);
					if((e1->urlId)>0 && uriNames[e1->urlId]!=NULL){
						QNamesVect->push_back(getPrefixName(i));
						QNamesVect->push_back(uriNames[e1->urlId]);
						QNamesVect->push_back(myMap->getData(e1->str_index));
					}
					else{
						QNamesVect->push_back("");
						QNamesVect->push_back("");
						QNamesVect->push_back(myMap->getData(e1->str_index));
					}
					break;
				}
			}
		}
		return QNamesVect;
	}

	
	unsigned int allocate(int uriId,const char* prefix,int lengthPrefix,const char* uri,
			      int lengthURI,const char* localName,int lengthName);

	unsigned int allocate(int uriId,MyVector<const char> * prefix,int lengthPrefix,MyVector<const char> * uri,
			      int lengthURI,MyVector<const char>* localName,int lengthName);

			
	unsigned int allocateCode(unsigned nc);
	unsigned int allocateNamespace(const char* prefix, int lengthPrefix,const char* uri, int lengthURI,int uriId);

	//space usage info
	double spaceUsage(){
		
		double num=(ncCap*4+uriCap*22+uriCap*4+prefixesSize*4*uriCap+QNamesSize*4)+myMap->spaceUsage();

		return num;
	}
	
	int getNameCodeSize(){return ncSize;}
 private:
	
	int QNamesSize;
	short uriUsed;
	short prefixesUsed;
	short freePrefix;
	short ncSize;
	short ncCap;
	short uriCap;
	short uriSize;
	short prefixesSize;
	string** prefixForUri;
	char** uriNames;
	char** QNames;
	unsigned int* nameCodeA;
	HashMap <>*myMap;
};
#endif
