/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef STREE_H
#define STREE_H

#include "STree/Paren.h"
#include "MyVector.h"

class STree{
public:
  STree(){}
  STree(int i){}
	
  STree(const STree &stree){
    tree_rep=stree.tree_rep;
    isTextNodeBV=stree.isTextNodeBV;
    nc=stree.nc;
    nullNode=stree.nullNode;
  }     
  STree(unsigned char *bs, int length, unsigned int depth){
    nullNode.node_n=0;
    nullNode.node_2n=0;

    tree_rep=new Paren(bs,(UINT)length, depth);
   
  }

  STree(MyVector<UBYTE> * bs, int length, unsigned int depth) {
    nullNode.node_n = 0;
    nullNode.node_2n = 0;

    tree_rep = new Paren(bs, (UINT)length, depth);
  }
  
  void setupTextNodeBV(unsigned char* isTextBV, int length){
    nc=length;
    //isTextNodeBV = new myCRank(isTextBV,length);
   isTextNodeBV = new tFID(isTextBV,length,suppSelect0);
  }

  void setupTextNodeBV(MyVector<UBYTE> * isTextBV, int length){
    nc=length;
    
   isTextNodeBV = new tFID(isTextBV,length,suppSelect0);
  }


  double spaceUsage(){
    
    return ((nc/4)+((nc/8)*1.5));
  }
  
  ~STree(){
    delete isTextNodeBV;
    delete tree_rep;
  }
  
 
	short comparePosition(nodeValue n, nodeValue other){
	  if(n.node_n<other.node_n)
	    return -1;
	  if(n.node_n>other.node_n)
	    return 1;
	  return 0;
	} 
	

	  	
	//updated 4/06/2010
	inline int getNodePosition(int i){
          tree_rep->rank0(i);
        }
	
	inline nodeValue firstChild(nodeValue n){return tree_rep->firstChild(n);}
	inline nodeValue lastChild(nodeValue n){return tree_rep->lastChild(n);}
	inline nodeValue parent(nodeValue n){return tree_rep->parent(n);}
	inline nodeValue nextSibling(nodeValue n){return tree_rep->nextSibling(n);}
	inline nodeValue previousSibling(nodeValue n){return tree_rep->previousSibling(n);}
	inline int descendants(nodeValue n){return tree_rep->descendants(n);}
	inline nodeValue nextNode(nodeValue n){return tree_rep->nextNode(n);}
	inline nodeValue previousNode(nodeValue n){return tree_rep->previousNode(n);}
	
	
	inline bool hasChildNodes(nodeValue i){
	  return tree_rep->hasChildren(i);
	}

	inline bool hasNextSibling(nodeValue i) {
	  return tree_rep->hasNextSibling(i);
	}
 
	nodeValue childrenCount(nodeValue n);
	nodeValue subTreeCount(nodeValue n);
	nodeValue child(nodeValue n,int i);

	
	inline int getNodeCount(){
	  return nc;
	}

	inline int getNodeNumber(nodeValue n){
	  return n.node_n;
	}
	
	inline nodeValue getOtherNodeValue(int i){
	  nodeValue myNode;
	  myNode.node_2n=selectOtherNode(i);
	  myNode.node_n=tree_rep->rank0(myNode.node_2n);
	  return myNode;
	}
		
	//counting from 1
	inline bool isTextNode(int n){
	  return isTextNodeBV->getInternal(n-1);
	}
	
	inline int getBit(int n){
	  return isTextNodeBV->getInternal(n-1);
	}

	inline int isTextRank0(int n){return isTextNodeBV->rank0(n);}
	
	inline int isTextRank1(int n){return isTextNodeBV->rank1(n);}
	
	//counting from 1
	inline int getTextNodeNumber(int n){
	  if(isTextNode(n))
	    return n-isTextNodeBV->rank0(n);
	  return -1;
	}
	
	inline int getOtherNodeNumber(int n){
	if(isTextNodeBV->getInternal(n-1))
	    return -1;
	  return isTextNodeBV->rank0(n);
	}
	
	inline int selectOtherNode(int i){
	for(int j=1;j<30;j++){
		cout<<"isTextNodeBV->select0(i): "<<j<<"="<<isTextNodeBV->select0(j)<<endl;
	}
	  return isTextNodeBV->select0(i);
	}
	
  tFID *myCFID;

	unsigned int getDoubleNumber(unsigned int paren_index) {
		return tree_rep->getDoubleNumber(paren_index);
	}

private:
	Paren *tree_rep;
	
	tFID *isTextNodeBV;
	int nc;
	nodeValue nullNode;
};
#endif
