/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */


#ifndef __PAREN_H
#define __PAREN_H
//

#include "../myUtils.h"
#include "../MyVector.h"

class Paren{

public:
  Paren(){}
  Paren(unsigned char *bv, int l, unsigned int depth);
  Paren(MyVector<UBYTE> * bv, int l, unsigned int depth); 
  ~Paren(){
    delete m_bv;
  }
  
  UINT rank0(UINT n){
    return m_bv->rank0(n);
  }
  
  
  inline int getNodePosition(int node_n){
    return rank0(node_n);
  }
  
  inline nodeValue FindClose(nodeValue openpdn)
  {
    nodeValue closepdn=nullNode;
    closepdn.node_2n = m_bv->FindClose(openpdn.node_2n);
	  
    if ( closepdn.node_2n!=-1 )
      {
	closepdn.node_n=openpdn.node_n + 
	  (closepdn.node_2n-openpdn.node_2n)/2;
      }
    else
      {
	closepdn.node_n = 0;
	closepdn.node_2n = 0;
      }
    
    return closepdn;
  } 
  
  
  inline nodeValue FindEnclose(nodeValue childpdn)
  {
    nodeValue enclosepdn=nullNode;
    enclosepdn.node_2n = m_bv->FindEnclose(childpdn.node_2n);
    if ( enclosepdn.node_2n!=-1 )
      {
	enclosepdn.node_n = childpdn.node_n -
	  (childpdn.node_2n-enclosepdn.node_2n+1)/2;
      }
    else 
      {
	enclosepdn.node_n = 0;
	enclosepdn.node_2n = 0;
      }
    return enclosepdn;
  }	
  
  
  inline nodeValue FindOpen(nodeValue closepdn)
  {
    nodeValue openpdn;
    openpdn.node_2n = m_bv->FindOpen(closepdn.node_2n);
    
    if ( openpdn.node_2n!=-1 )
      {
	openpdn.node_n = closepdn.node_n -
	  (closepdn.node_2n-openpdn.node_2n)/2;
      }
    else 
      {
	openpdn.node_n = 0;
	openpdn.node_2n = 0;
      }
    return openpdn;
  }

  bool hasChildNodes(UINT NodeOpen);
  bool hasChildren(nodeValue i);
  bool hasNextSibling(nodeValue i);
  int descendants(nodeValue i); // new added method
  
  nodeValue parent(nodeValue i);
  nodeValue child(nodeValue x, int i);
  nodeValue firstChild(nodeValue i);
  nodeValue lastChild(nodeValue i);
  nodeValue nextSibling(nodeValue i);
  nodeValue previousSibling(nodeValue i);
  nodeValue nextNode(nodeValue i);
  nodeValue previousNode(nodeValue i);
  
  double spaceUsage(){return 0;}

  unsigned getDoubleNumber(unsigned int paren_index);
  
private:
  tPAREN * m_bv;
  nodeValue nullNode;
  int k; //count of nodes in the tree
  int nc; //nodeCount (improvement for the NextOpen operation)
  unsigned int maxDepth;
};
#endif // __PAREN_H
