/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef __SUCTREE_H
#define __SUCTREE_H

#include "NameCodeDS.h"
#include "AttributeDS.h"
#include "STree.h"
#if defined(TEXTDS)
  #include "TextDS.h"
#else
  #include "TextBZDS.h"
#endif
#include "SDOM_Utils.h"
#include "DocType.h"

class SucTree{


public:
  SucTree(){
  }
  
  //constructor - adding a SDOM internal structure
  SucTree(TextDS  *txDS, NameCodeDS *nCodeDS,  STree*tree,  AttributeDS *attrDS, DocType *dType){
    textDS=txDS;
    myNameCodeDS=nCodeDS;
    sTree=tree;
    myAttributeDS=attrDS; 
    docType=dType; 
  }
  SucTree(const SucTree& mySucTree)throw(bad_alloc){
    nodeCount=mySucTree.nodeCount; 
    docCount=mySucTree.docCount;
    cap=mySucTree.cap;
      textDS=textDS;
      myNameCodeDS=myNameCodeDS;
      sTree=sTree;
      myAttributeDS=myAttributeDS;
      docType=docType;
  }
  
  //destructor
  ~SucTree(){
      delete textDS;
      delete myNameCodeDS;
      delete sTree;
      delete myAttributeDS;
	if (docType != NULL)
	      delete docType;
	delete [] actualEncoding;
	delete [] encoding;
  }

	static void release() {
		CRank<Simple_block<RB1>,RB1>::release();
		tPAREN::release();

	}
  
  inline void setDocId(short id){docId=id;}
   
  //add a new document to the SucTree structure
  void addDoc(TextDS  *txDS, NameCodeDS *nCodeDS,  STree *tree,  AttributeDS *attrDS, DocType *dType){
     
      textDS=txDS;
      myNameCodeDS=nCodeDS;
      sTree=tree;
      myAttributeDS=attrDS;  
      docType=dType;
      docCount++;   
   }
   
   //set the document attributes refrenced by the docId
   void setDocumentInfo(bool isStrictErr, bool isStdalone, double ver, char* actEncode, char* encode, const char* doc){
     						isStrictErrCheck=isStrictErr;
     Standalone=isStdalone;
     version=ver;   
     actualEncoding=actEncode;
     encoding=encode;
     int myLen=0;
     docURI=doc;
  }
  
  //return: the Text data structure according to the docId in position
  TextDS &getTextDS(){return *textDS;}
  
  //return: the NameCode data structure according to the docId in position
  NameCodeDS &getNameCodeDS(){return *myNameCodeDS;}
  
  //return: the Succinct Tree data structure according to the docId in position
  STree &getStree(){return *sTree;}
  
  //return: pointer to the Succinct Tree data structure according to the docId in position
  STree *getStree2(){return sTree;}
  
  //return: the Attribute data structure according to the docId in position
  AttributeDS &getAttributeDS(){return *myAttributeDS;}
  
  //return: the Namepool data structure according to the docId in position
  NamePool &getNamepool(){return *(myNameCodeDS)->getNamePool();}
   
  //return: the DocType data structure according to the docId in position
  DocType &getDocType(){return *docType;}
  
   
   
  double spaceUsage(){
    cout<<"TextDS ="<<(textDS->spaceUsage())<<" B, ";
    double num=textDS->spaceUsage();
    cout<<endl;
    num+=myNameCodeDS->spaceUsage();  
    cout<<"NameCodeDS ="<<myNameCodeDS->spaceUsage()<<" B, ";
    cout<<endl;
    num+=myAttributeDS->spaceUsage();
    cout<<"AttributeCodeDS ="<<myAttributeDS->spaceUsage()<<" B, ";
    cout<<endl;
    num+=sTree->spaceUsage(); 
    cout<<"sTree ="<<sTree->spaceUsage()<<" B, ";
    cout<<endl;
    if(docType!=NULL){
      cout<<"DocType ="<<docType->spaceUsage()<<" B, ";
      cout<<endl;
      num+=docType->spaceUsage();  
    }
    cout<<endl<<" Total:="<<num/1024<<" KB"<<endl;
    return num;
  }

 inline bool getErrCheck(){return isStrictErrCheck;}
 inline bool isStandalone(){return Standalone;}
 inline double getVersion(){return version;}
 inline char* getActualEncoding(){return actualEncoding;}
 inline char* getEncoding(){return encoding;}
 const char* getdocURI(){return docURI;}
 
 short docId;
 
  
private:
 TextDS  *textDS;
  NameCodeDS *myNameCodeDS;
  STree *sTree;
  AttributeDS *myAttributeDS;
  DocType *docType;
  bool isStrictErrCheck;
  bool Standalone;
  double version;
  char * actualEncoding;
  char* encoding;
  const char* docURI;
  int nodeCount; 
  short docCount; 
  short cap; 
};
#endif
