/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 

#ifndef SDOM_Exception_
#define SDOM_Exception_



class SDOM_Exception  {
public:
    // -----------------------------------------------------------------------
    //  Constructors
    // -----------------------------------------------------------------------
    /** @name Constructors */
    //@{
    /**
      * Default constructor for SDOM_Exception.
      *
      */
    SDOM_Exception();

    /**
      * Constructor which takes an error code and a message.
      *
      * @param code           The error code which indicates the exception
      * @param message        The string containing the error message
      * @param memoryManager  The memory manager used to (de)allocate memory
      */
    SDOM_Exception(      
                       short                 code
               , const char*                message
                );

    /**
      * Copy constructor.
      *
      * @param other The object to be copied.
      */
    SDOM_Exception(const SDOM_Exception &other);

    //@}

    // -----------------------------------------------------------------------
    //  Destructors
    // -----------------------------------------------------------------------
    /** @name Destructor. */
    //@{
	 /**
	  * Destructor for SDOM_Exception.
	  *
	  */
    virtual ~SDOM_Exception();
    //@}

public:
   /*
     * The above is since DOM Level 2 @since DOM Level 3
     */
    enum ExceptionCode {
         INDEX_SIZE_ERR       = 1,
         DOMSTRING_SIZE_ERR   = 2,
         HIERARCHY_REQUEST_ERR = 3,
         WRONG_DOCUMENT_ERR   = 4,
         INVALID_CHARACTER_ERR = 5,
         NO_DATA_ALLOWED_ERR  = 6,
         NO_MODIFICATION_ALLOWED_ERR = 7,
         NOT_FOUND_ERR        = 8,
         NOT_SUPPORTED_ERR    = 9,
         INUSE_ATTRIBUTE_ERR  = 10,
         INVALID_STATE_ERR    = 11,
         SYNTAX_ERR     = 12,
         INVALID_MODIFICATION_ERR    = 13,
         NAMESPACE_ERR     = 14,
         INVALID_ACCESS_ERR   = 15,
         VALIDATION_ERR       = 16
        };
    //@}

    // -----------------------------------------------------------------------
    //  Getter
    // -----------------------------------------------------------------------
    inline const char* getMessage()    const;

    // -----------------------------------------------------------------------
    //  Class Types
    // -----------------------------------------------------------------------
    /** @name Public variables */
    //@{
	 /**
	  * A code value, from the set defined by the ExceptionCode enum,
     * indicating the type of error that occured.
     */
    ExceptionCode   code;

	 /**
	  * A string value.  Applications may use this field to hold an error
     *  message.  The field value is not set by the DOM implementation,
     *  meaning that the string will be empty when an exception is first
     *  thrown.
	  */
    const char *msg;
    //@}

protected:


private:

	 /**
	  * A boolean value.  
      *   If the message is provided by the applications, it is not 
      *   adopted.
      *   If the message is resolved by the DOM implementation, it is
      *   owned.
	  */
    bool            fMsgOwned;

private:
    // -----------------------------------------------------------------------
    // Unimplemented constructors and operators
    // -----------------------------------------------------------------------    
    SDOM_Exception & operator = (const SDOM_Exception &);
};

inline const char* SDOM_Exception::getMessage() const
{
    return msg;
}


#endif


