/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 
#ifndef _SDOM_NODE_LIST_H
#define _SDOM_NODE_LIST_H  

#include "SDOM_Node.h"
#include "../Internal2/SucTree.h"

class SDOM_Node;

class SDOM_NodeList{

public:
  
  /**
  * Default constructor
  */
  SDOM_NodeList();

   
  /**
  * Main constructor
  * @param  Pointer to internal SDOM, array of node IDs and array length
  */
    SDOM_NodeList(SucTree *suc,nodeValue* arr, int l);
//    SDOM_NodeList(NodeListImpl *list);
  
  /**
  * Copy constructor
  * @param  SDOM_NodeList object
  */
  SDOM_NodeList(const SDOM_NodeList &nlist);

  /**
  * Destructor
  */
   ~SDOM_NodeList(){}
   
   /**
     * Returns the <code>index</code> item in the collection.
     *
     * If <code>index</code> is greater than or equal to the number of nodes in
     * the list, this returns <code>null</code>.
     *
     * @param index Index into the collection.
     * @return The node at the <code>index</code>th position in the
     *   <code>SDOM_NodeList</code>, or <code>null</code> if that is not a valid
     *   index.
     * @since DOM Level 1
     */
   SDOM_Node  *item(int index);
   
   /**
     * Returns the number of nodes in the list.
     *
     * The range of valid child node indices is 0 to <code>length-1</code> inclusive.
     * @since DOM Level 1
   */
  int getLength()const;

 
private:
    SucTree *sucTree; /** Pointer to the core internal SDOM representation */
      nodeValue* items;/** Array of node IDS in list, represented as  2 integers */
      int length; /** Length of node array */

};
#endif
