/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 
#ifndef SIXML_XERCES_PARSER_H_
#define SIXML_XERCES_PARSER_H_

#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/parsers/XercesDOMParser.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/internal/XMLScannerResolver.hpp>
#include <xercesc/sax/HandlerBase.hpp>
#include <xercesc/util/XMLChar.hpp>
#include <xercesc/util/PlatformUtils.hpp>
#include <string>    
#include <fstream> //fstream     
#if defined(XERCES_NEW_IOSTREAMS)
#include <iostream>
#else
#include <iostream.h>
#endif
#include <time.h>
#include <stdio.h>

#if defined(JNI)
#include "../../myInterface/MyNodeImpl.h"
#else
#include "../../SIXDOM/SDOM_Node.h"
#endif
#include "../../Internal2/SucTree.h"

XERCES_CPP_NAMESPACE_USE
using namespace std;

class SucTree;
class TextDS;
class NameCodeDS;
class STree;
class AttributeDS;
class DocType;
template <typename T>class  MyVector;


class SIXDOM_Parser : public XercesDOMParser{
public:
	unsigned int currentText1;
	NTextDS * ds;
	vector<char *> v;
	TextBlock * tb;
	unsigned long totalChars1;
	unsigned long int sizes[64];


	unsigned long maxChars;
	unsigned long minChars;
	unsigned long blockSize;
	unsigned long currentCharBlock;
	unsigned long currentChars;
	unsigned long totalChars;
	unsigned long numberOfBlocks;

  SIXDOM_Parser();
  virtual ~SIXDOM_Parser(){}
  STree &getTree();


  int parseString(const char* xml_str){return 0;}

  char* convertChars(const XMLCh* mychars);
   virtual void docCharacters (const XMLCh *const chars, const unsigned int length, const bool cdataSection);
  void attrDataHelper(const XMLCh *chars, int length);
  void CharacterDataHelper(const XMLCh *chars, int length);

 virtual void docComment (const XMLCh *const comment);


 virtual void docPI (const XMLCh *const target, const XMLCh *const data); 


 virtual void endDocument ();

 virtual void endElement (const XMLElementDecl &elemDecl, const unsigned int urlId, const bool isRoot, const XMLCh *const elemPrefix);


  virtual void endEntityReference (const XMLEntityDecl &entDecl);


 virtual void ignorableWhitespace (const XMLCh *const chars, const unsigned int length, const bool cdataSection);

 virtual void resetDocument ();

 virtual void startDocument ();


 virtual void startElement (const XMLElementDecl &elemDecl, const unsigned int urlId, const XMLCh *const elemPrefix, const RefVectorOf< XMLAttr > &attrList, const unsigned int attrCount, const bool isEmpty, const bool isRoot);

 virtual void startEntityReference (const XMLEntityDecl &entDecl);


 virtual void XMLDecl (const XMLCh *const versionStr, const XMLCh *const encodingStr, const XMLCh *const standaloneStr, const XMLCh *const actualEncStr);

 virtual void elementTypeInfo (const XMLCh *const typeName, const XMLCh *const typeURI);

//doctypehandler interfaces
 virtual void attDef (const DTDElementDecl &elemDecl, const DTDAttDef &attDef, const bool ignoring);


 virtual void doctypeComment (const XMLCh *const comment);


 virtual void doctypeDecl (const DTDElementDecl &elemDecl, const XMLCh *const publicId, const XMLCh *const systemId, const bool hasIntSubset, const bool hasExtSubset=false);


 virtual void doctypePI (const XMLCh *const target, const XMLCh *const data);


 virtual void doctypeWhitespace (const XMLCh *const chars, const unsigned int length);


 virtual void elementDecl (const DTDElementDecl &decl, const bool isIgnored);


 virtual void endAttList (const DTDElementDecl &elemDecl);


 virtual void endIntSubset ();


 virtual void endExtSubset ();


 virtual void entityDecl (const DTDEntityDecl &entityDecl, const bool isPEDecl, const bool isIgnored);


 virtual void resetDocType ();


 virtual void notationDecl (const XMLNotationDecl &notDecl, const bool isIgnored);


 virtual void startAttList (const DTDElementDecl &elemDecl);


 virtual void startIntSubset ();


 virtual void startExtSubset ();


 virtual void TextDecl (const XMLCh *const versionStr, const XMLCh *const encodingStr);
    
private:
  private:
    
    bool entDebug;
    
  //document-order representations
  void setBS_Bit(MyVector<unsigned char>* bsVector,unsigned char &current, int &pos, int &size,const unsigned char bit);
    
      
  unsigned char* getChars(MyVector<unsigned int>* bsVector);
    
  unsigned char* getChars(MyVector<unsigned char>* bsVector);

  void myAttributes(const RefVectorOf< XMLAttr > &attrList, const unsigned int attrCount);

  bool lineBreakFlag;
  int uriCount, cdata_length,textn_length;
  int treePos, treeSize,textBVSize, textPos, text_index, nodeCount, attLength,elemCount;
  int depth,textNodePos, textNodeSize,levelOrderSize,mtext_length,ntext_count,mAttr_length,nAttr_count, m, m2,textBufPos, BlockSize,BlockSize2, nsElemStackCount;;
  unsigned char currentText, currentParan;
  bool foundER;
  bool endOfDoc;
  int CountAttrForElements;
  unsigned char * currentBlock;
  unsigned char * currentAttrBlock;
  const char* fileName;
  int blockIndex, attrBlockIndex;
  MyVector<char>* newName;  
  MyVector<char>* newUrlName;
  MyVector<char>* newPrefix;
  MyVector<char>* newValue;
  MyVector<char>* newURL;
  unsigned char *isLeaf_Array;
  MyVector<unsigned char>*treeDF_BV;
  MyVector<unsigned char>*isTextNodeBV;
  MyVector<unsigned int>* text_offsets;
  MyVector<unsigned int>*attr_offsets;
  MyVector<unsigned char>*attCount_Vector;
  MyVector<unsigned char*>*attrTextBlocks;

  SucTree *sucTree;
  TextDS *textDS;
  NameCodeDS *myNameCodeDS;
  STree *sTree;
  AttributeDS *myAttributeDS;
  DocType *myDocType;
  

  unsigned int currentDepth;
  unsigned int maxDepth;  
};
#endif

