/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Naila Rahman, Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef __CFID_H
#define __CFID_H

#include "utils.h"
#include "buffer.h"
#include "CRank.h"


#ifndef SSB
#define SSB             32            // Select. Blocking factor 
#endif

#if SSB==16
#define lgSSB            4
#define SSB_OFFSET(i)   ((i)&0x0F)
#elif SSB==32
#define lgSSB            5
#define SSB_OFFSET(i)   ((i)&0x01F)
#endif


#ifndef LARGE_GAP
//#define LARGE_GAP       128
#define LARGE_GAP       256
#endif




template <class TCRank, UINT R1> class CFID : public TCRank
{
public:
	static string name() { return string("CFID"); }
	static void release() {
		

	}

	CFID<TCRank,R1>(const UBYTE * data,const UINT n,const Select_e suppSel);
        CFID<TCRank,R1>(MyVector<UBYTE> * data,const UINT n,const Select_e suppSel);
	CFID<TCRank,R1>(const UINT  * data,const UINT n,const Select_e suppSel);
	~CFID<TCRank,R1>() {
		delete [] m_B0;
		delete [] m_B1;
		delete [] m_C;
		delete m_V;
	}


	CFID<TCRank,R1>(const CFID<TCRank,R1> & b1);
	CFID<TCRank,R1>& operator= (const CFID<TCRank,R1>& b1);

	UINT select0(const UINT i);
	UINT select1(const UINT i);
	void set(const UINT  * uData, const UINT n, const Select_e suppSel );
	void set(const UBYTE * uData, const UINT n, const Select_e suppSel );
	double spaceUsage(){
	  double num=spaceusage+TCRank::m_n+TCRank::m_n*0.5;
	  return num;
	}

private:
	UINT * m_B0;
	UINT * m_B1;
	UINT * m_C;			// assume C is not compressed
	TCRank  * m_V;
	UINT   m_n0;		// number of 0 bits
	UINT   m_n1;		// number of 1 bits
	UINT   m_nC;		// number of ints in index (address) array C
	UINT   m_nV;		// number of bits in vector V
	UINT   R1_LG;
	UINT   R1_MASK;
	double spaceusage;

	void copyobj(const CFID<TCRank,R1> * pb1);
		
	void AllocCopyArray(UINT ** array1, UINT * pn1, 
					const UINT * array2, const UINT n2);
	void AllocCopyArrayV(TCRank ** ppV1, UINT * pn1, 
			TCRank *  pV2,  UINT n2 );
	void InitB1();
	void InitB0();
	void InitB1BF();
	void InitB0BF();
	void InitVandC();
	void CheckVandC(UINTBuffer * pAddList, UINT nMaxItems);
	UINT select0NonLG(const UINT i);
	UINT select1NonLG(const UINT i);
	void GetStats1(UINT &OneCount, UINT &lastblockWith1,UINT &last1);
	void GetStats0(UINT &ZeroCount,UINT &lastblockWith0,UINT &last0);
};


#endif // __CFID_H
