/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Naila Rahman, Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef __CRANK_H
#define __CRANK_H

#include "utils.h"
#include "buffer.h"
#include "../Internal2/MyVector.h"

#define RE_FROMLE(i)      (i==1? 0 : LE(i-1) )
#define RE(i)             ((rank1(i)<<1) - (i) )
#define LE_FROMRE(i)      (i==1? 0 : RE(i-1) )


#ifdef USE_VIRTUAL_FUNC
#define VIRTUAL virtual
#else
#define VIRTUAL
#endif


extern UBYTE * aBmpTo1BitCount;
extern UBYTE * aBmpTo0BitCount;
extern BmpToBitOffset_t * aBmpToBitOffset;
extern BmpToBitOffset_t * aBmpToBit0Offset;
extern SubBlock_t aTopX1BitMask[RB2+1];



template <class T1, UINT R1> class CRank
{
public:
	static string name() { return string("CRank"); }
	static void release() {
		delete [] aBmpTo1BitCount;
		delete [] aBmpTo0BitCount;
		delete [] aBmpToBitOffset;
		delete [] aBmpToBit0Offset;
	}

        CRank<T1,R1>();
	CRank<T1,R1>(const UBYTE * data, const UINT n );
	CRank<T1,R1>(const UINT * data, const UINT n );
	CRank<T1,R1>(MyVector<UBYTE> * data, const UINT n );
	~CRank<T1,R1>() {
		delete [] m_bvR;
	}

	// For saftey declared but not defined.
	CRank<T1,R1>(const CRank<T1,R1> & b1);
	CRank<T1,R1>& operator= (const CRank<T1,R1>& b1);

	void get(UBYTE ** data, UINT * n);
	inline UINT get(const UINT offset) { return getInternal( offset-1 ); }
	//start from one
	UINT getNextOpen(const UINT offset);

	inline UINT rank0(const UINT i) { return i - rank1(i); }
	UINT rank1(UINT i);
	inline SubBlock_t * getSubBlock(UINT bi, UINT sbi)
	{
		return (m_bvR[bi].bits)[sbi];
	}
	inline UINT GetBlockCount( ) { return (m_n+R1-1)/R1; }

	inline UINT calcdeltaLE(SubBlock_t bmp) { return ((aBmpTo1BitCount[bmp]<<1) - RB2); }
	inline UINT calcdeltaRE(SubBlock_t bmp) { return (RB2 - (aBmpTo1BitCount[bmp]<<1)); }
	inline UINT GetSize() { return m_n; }
	VIRTUAL void SetRankBlockPrefixes(UINT n);
	VIRTUAL inline UINT GetRankBlockPrefix(UINT b) { return m_bvR[b].start_rank; }
	VIRTUAL inline UINT GetRank0BlockPrefix(UINT b) 
						{ return KRANK_BLOCK_TO_ADDR(b,R1_LG) - m_bvR[b].start_rank; }
	void InitBitsRank(const UBYTE * array2);
	void InitBitsRank(MyVector<UBYTE> * array2);
	UINT getInternal(const UINT offset);

	/*
	** note we are exposing the main data !
	*/
	T1    * m_bvR;		// bit vector and rank
	UINT   m_n;			// number of bits in vector
	UINT   m_n1;
	UINT R1_LG;
	UINT R1_MASK;

private:

	void copyobj(const CRank<T1,R1> * pb1);
	void AllocCopyBitsRank( MyVector<UBYTE> * array2, const UINT n2);		
	void AllocCopyBitsRank( const UBYTE * array2, const UINT n2);
	void AllocCopyBitsRank( const T1 * pbvR2, const UINT n2);
};


template <UINT R1> class Simple_block
{
public:
	UINT   start_rank;
	SubBlock_t  bits[R1/(sizeof(SubBlock_t)*8)];
};



template <class T> void dumpbinary( T * bvR, UINT nbits);

#endif // __CRANK_H
