/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Naila Rahman, Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef _CSIMPLEOPTTREE_H
#define _CSIMPLEOPTTREE_H

#include "CRank.h"
#include "CFID.h"
#include "CUnaryFID.h"
#include "CSimpleFID.h"
#include "utils.h"
#include "buffer.h"
#include "../Internal2/MyVector.h"

#ifdef COMPRESS_RANK
#define CRANK CRank2
#else
#define CRANK CRank
#endif


#ifdef USE_UNARYFID
#define FID_TYPE CUnaryFID< CFID<CRank<Simple_block<R1>, R1>, R1> >
{
#else
#define FID_TYPE CSimpleFID<R1>
#endif


extern int DoubleSearchCount;
extern int SearchCount;

extern BmpParenMap_t * pSOTBmpOpenParenMatcher;
extern BmpParenMap_t * pSOTBmpCloseParenMatcher;
extern BmpParenMap_t * pSOTBmpOpenParenMatchHiToLo;
extern BmpParenMap_t * pSOTBmpCloseParenMatchLoToHi;
extern BmpE_t * pSOTBmpMaxRE;
extern BmpE_t * pSOTBmpMaxLE;
extern BmpE_t * pSOTBmpMinRE;
extern BmpE_t * pSOTBmpMinLE;


template <class TCRank, UINT R1> class CSimpleOptTree : public TCRank
{
public:
	static string name() {return string("CSimpleOptTree"); }

	static void release() {
		delete [] pSOTBmpCloseParenMatcher;
	       	delete [] pSOTBmpMinRE;
	       	delete [] pSOTBmpCloseParenMatchLoToHi; 
	       	delete [] pSOTBmpMaxRE;
       		delete [] pSOTBmpOpenParenMatcher;
		delete [] pSOTBmpMinLE;
		delete [] pSOTBmpOpenParenMatchHiToLo;
       		delete [] pSOTBmpMaxLE;
	}

	CSimpleOptTree<TCRank,R1>();
	CSimpleOptTree<TCRank,R1>(const UBYTE * data, const UINT n);
	CSimpleOptTree<TCRank,R1>(MyVector<UBYTE> * data, const UINT n);
	CSimpleOptTree<TCRank,R1>(const UBYTE * data, const UINT n, UINT recLevel);
	CSimpleOptTree<TCRank,R1>(const UINT * data, const UINT n);
	~CSimpleOptTree<TCRank,R1>() {
		delete m_PFlagsV;
		delete m_PFVr;
		delete[] m_PFVb;
		delete[] m_PFVbEnclose;
	}

	//CSimpleOptTree<TCRank,R1>(const CSimpleOptTree<TCRank,R1> & b1);
	//CSimpleOptTree<TCRank,R1>& operator= (const CSimpleOptTree<TCRank,R1>& b1);
	
	/*inline nodeValue FindClose(nodeValue openpdn)
        {
		nodeValue closepdn;
		closepdn.node_2n = FindClose(openpdn.node_2n);
        	if ( closepdn.node_2n!=-1 )
        	{
                	closepdn.node_n=openpdn.node_n + 
                                    (closeclosepdn.node_2n-openpdn.node_2n)/2;
        	}
        	else
        	{
            		closepdn.node_n = 0;
			closepdn.node_2n = 0;
        	}

        	return closepdn;
        }


	inline nodeValue FindEnclose(nodeValue childpdn)
	{
		nodeValue enclosepdn;
		enclosepdn.node_2n = FindEnclose(childpdn.node_2n);
		if ( enclosepdn.node_2n!=-1 )
		{
			enclosepdn.node_n = childpdn.node_n -
                                   (childpdn.node_2n-enclosepdn.node_2n+1)/2;
		}
		else 
		{
            		enclosepdn.node_n = 0;
			enclosepdn.node_2n = 0;
		}

		return enclosepdn;
	}	


	inline nodeValue FindOpen(nodeValue closepdn)
	{
		nodeValue openpdn;
		openpdn.node_2n = FindOpen(closepdn.node_2n);

		if ( openpdn!=-1 )
		{
			openpdn.node_n = closepdn.node_n -
                                       (closepdn.node_2n-openpdn.node_2n)/2;
		}
		else 
		{
            		openpdn.node_n = 0;
			openpdn.node_2n = 0;
		}

		return openpdn;
	}*/
	
	UINT FindClose(UINT i);
	UINT FindEnclose(UINT i);
	UINT FindOpen(UINT i);
	//void TestNearOpenClose();
	UINT getNextOpen( const UINT startAddr);
	UINT getPreviousOpen( const UINT startAddr );
		

private:
// 	UINT FindClose(UINT i);
// 	UINT FindEnclose(UINT i);
// 	UINT FindOpen(UINT i);
// 	void TestNearOpenClose();

	UINT   R1_LG;
	UINT   R1_MASK;
	UINT   m_FirstFindEnclose;
	UINT * m_pMatches;
	UINT * m_pEnclose;
	UINT * m_anode_n1Prefix;
	UINT   m_recLevel;


	FID_TYPE        * m_PFlagsV;
UINT * aAddr2;

CUnaryFID< CFID<CRank<Simple_block<R1>, R1>, R1> > * m_PFlagsVTester;

	UINT            * m_PFVb;
	UINT		* m_PFVbEnclose;
	CSimpleOptTree<TCRank,R1>  * m_PFVr;
	UINT              m_fb;
	SubBlock_t      * m_pfbv;
	UBYTE			* m_IsFar;
	
	
		
	void FindMatches();
	void BuildPioneers();
	void BuildFlagVector(UINT * aAddresses, UINT nPioneers);
	void BuildPioneerFamilyVector(UINT * aAddresses, UINT nPioneers);
	void SetupLowerMinLEBlockNo();

	UINT NearEncloseSearch( UINT i );
	UINT FarEncloseSearch( UINT i );

	UINT FarCloseSearch( UINT i );
	UINT FarCloseSearch(UINT i, UINT PioneerOpen, UINT PioneerClose );
	void FindPioneerFromOpen(UINT i,UINT *pPioneerOpen,UINT *pPioneerClose);
	UINT SearchDeltaRE( UINT PioneerClose, int deltaLE );
	UINT GetDeltaLE( UINT i, UINT PioneerOpen );
	UINT NearCloseSearch( UINT i, SubBlock_t *pbv );
	UINT NearOpenSearch( UINT i, SubBlock_t *pbv, bool fCheckLastBitPrevBlock );
	inline UINT IsClose(const UINT i) { return TCRank::getInternal(i); } // 0,..,n-1
	UINT FarOpenSearch( UINT i );
	UINT GetDeltaRE( UINT i, UINT PioneerClose );
	UINT SearchDeltaLE( UINT PioneerOpen, int deltaLE );
	UINT FarOpenSearch(UINT i, UINT PioneerOpen, UINT PioneerClose);
	void FindPioneerFromClose( UINT i, UINT *pPioneerOpen, UINT *pPioneerClose);
	void DumpStats(UINT i, BOOL fDumpBlock);
};

#endif // _CSIMPLEOPTTREE_H
