/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Naila Rahman, Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */
 

#ifndef __BUFFER_H
#define __BUFFER_H


#include "utils.h"

template <class T> class Buffer
{
public:
	~Buffer()	  { delete m_pData; }
	Buffer(int n) 
	{
		m_n=n; 
		m_pData=new T[m_n]; 
		m_nItems=0; 
		m_Head=0;
		m_Tail=0;
	}
	void Add( T item)
	{	
		m_pData[m_Tail] = item;
		m_Tail++;
		if ( m_Tail == m_n ) 
		{
			m_Tail = 0;
		}

		if ( m_nItems == m_n )
		{
			m_Head++;
			if ( m_Head == m_n ) m_Head = 0;
		}

		if ( m_nItems < m_n )
		{
			m_nItems++;
		}
	}

	void Get( T * pData )
	{
		UINT i, j;
		
		j = 0;
		if ( m_Head >= m_Tail )
		{
			for ( i=m_Head; i<m_n; i++, j++ )
			{
				pData[j] = m_pData[i];
			}

			for ( i=0; i<m_Tail; i++, j++ )
			{
				pData[j] = m_pData[i];
			}
		}
		else
		{
			for ( i=m_Head; i<m_Tail; i++, j++ )
			{
				pData[j] = m_pData[i];
			}
		}
	}

	void Get( T ** pData, UINT * n )
	{
		*pData = new T[ m_nItems ];
		*n     = m_nItems;
		Get( *pData );
	}

	UINT GetCount()
	{
		return m_nItems;
	}

	T GetFirst()
	{
		return m_pData[m_Head];
	}

	T GetLast()
	{
		if ( m_Tail== 0 )
			return m_pData[m_n-1];
		else
			return m_pData[m_Tail-1];
	}

	void DeleteHead( )
	{
		if ( m_nItems > 0 )
		{
			m_Head++;
			if ( m_Head == m_n ) m_Head = 0;

			m_nItems--;
		}
	}

	void Flush()
	{
		m_Head   = 0;
		m_Tail   = 0;
		m_nItems = 0;
	}

private:
	Buffer(const Buffer & b1);
	Buffer& operator= (const Buffer& b1);

	T * m_pData;
	UINT m_n;
	UINT m_nItems;
	UINT m_Head;
	UINT m_Tail;
};



typedef Buffer<UBYTE> UBYTEBuffer;
typedef Buffer<UINT>  UINTBuffer;


#endif // __BUFFER_H
