/* Copyright Feb 2013 O'Neil Delpratt, Stelios Joannou and Naila Rahman, Rajeev Raman
 * University of Leicester
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 */

#ifndef __UTILS_H
#define __UTILS_H

#include <iostream>

#include <string.h>
using namespace std;
#include "../Internal2/MyVector.h"

#ifdef _DEBUG
#include <assert.h>
#define ASSERT    assert
#else
#include <assert.h>
#define ASSERT
#endif

#define BYTE    char
#define UBYTE   unsigned char
#define UCHAR   unsigned char
#define UINT    unsigned long int 
#define USHORT  unsigned short


#define RB3	8

#if  RB2==8
	#define SUBBLOCKMASK		0x07
	#define SUBBLOCKSHIFT		3
	typedef unsigned char SubBlock_t;
	typedef unsigned char SSubBlock_t;
#else
	#define SUBBLOCKMASK		0x0F
	#define SUBBLOCKSHIFT		4
	typedef unsigned short SubBlock_t;
	typedef unsigned char  SSubBlock_t;
#endif

#define SUBBLOCKCOUNT(R1)		((R1)/RB2)

#define MASK_R5    0x01F
#define MASK_R6    0x03F
#define MASK_R7    0x07F
#define MASK_R8    0x0FF
#define MASK_R9    0x01FF
#define MASK_L8    0xFF000000
#define MASK_R16   0x0FFFF
#define MASK_L16   0xFFFF0000
#define MASK_R24   0x0FFFFFF
#define MASK_L24   0xFFFFFF00

#define LD24(a)  ((((UINT)  a)>>8))
#define RU8(a)   ((((UINT)  a)<<8))

#if RB1==32
	#define lgRB1									5
	#define RB1_MASK								MASK_R5
#elif RB1==64
	#define lgRB1									6
	#define RB1_MASK								MASK_R6
#elif RB1==128
	#define lgRB1									7
	#define RB1_MASK								MASK_R7
#elif RB1==256
	#define lgRB1									8
	#define RB1_MASK								MASK_R8
#elif RB1==512
	#define lgRB1									9
	#define RB1_MASK								MASK_R9
#endif

#define KRANK_BLOCK(i,lgBS)                 ((((UINT)  i)>>lgBS))
#define KFULLADDR(b,o,lgBS)                 (((((UINT) b)<<lgBS)) | (o))
#define KRANK_BLOCK_TO_ADDR(b,lgBS)         ((((UINT)  b)<<lgBS))

#define KRANK_BLOCK_OFFSET(i,BMASK)         ((UINT)(i) & BMASK)
#define KRANK_BLOCK_OFFSET_TO_SUBBLOCK(i)   ((UINT)(i)>>SUBBLOCKSHIFT)		
#define KRANK_SUBBLOCK(i,BMASK)             (((UINT)(i) & BMASK)>>SUBBLOCKSHIFT)
#define KRANK_SUBBLOCK_OFFSET(i)            ((i)&SUBBLOCKMASK)


#ifdef EXCESS_ROW 
#define PMDEREF(T,b,e)		T[(b)].E[(e)]
#else
#define PMDEREF(T,b,e)		T[(e)].B[(b)]
#endif

#define MIN(a, b)  ((a)<(b) ? (a) : (b))


typedef enum { FALSE=0, TRUE=1 } BOOL;


typedef struct
{
	UBYTE offset[RB2];
} BmpToBitOffset_t;


typedef struct
{
	BYTE E;
	BYTE dE;

} BmpE_t;



typedef struct
{
#ifdef EXCESS_ROW
	UBYTE E[RB3];
#else
	UBYTE B[1<<RB3];
#endif

} BmpParenMap_t;


#define GET_NEARADDRESS(pi)						(pi.NearAddress)
#define GET_FARADDRESS(pi)						(pi.FarAddress)
#define SET_NEARADDRESS(pi, v)					((pi.NearAddress) = (v))
#define SET_FARADDRESS(pi, v)					((pi.FarAddress)  = (v))



typedef struct {

	UINT  NearAddress;
	UINT  FarAddress;

} pioneer_info3;

typedef enum
{
        suppSelect0 =0x001, 
        suppSelect1 =0x002,
        suppSelect01=0x003,

} Select_e;



#define UNCOMPRESSIDX(uiAcompress, usAcompress, idx ) \
	(uiAcompress[(idx >> 3) + ((idx >> 3) << 2)]) \
	+ (usAcompress[idx + (((idx >> 3) + 1) << 1)]) \


void InitBmpToVal( UBYTE ** ppBmpTo1BitCount, SubBlock_t * pTopX1BitMask );
void InitBmpToVal( UBYTE ** ppBmpTo1BitCount, UBYTE ** ppBmpTo0BitCount,
				SubBlock_t * pTopX1BitMask );
void InitBmpToBitOffset( BmpToBitOffset_t ** pBmpToBitOffset );
void InitBmpToBit0Offset(BmpToBitOffset_t ** ppBmpToBit0Offset );
void InitOpenParenMap(BmpParenMap_t ** ppBmpParenMap, BmpE_t ** pBmpMinLE );
void InitCloseParenMap(BmpParenMap_t ** ppBmpParenMap, BmpE_t ** pBmpMinRE );
void InitOpenParenMatchHiToLo(BmpParenMap_t ** ppBmpParenMap, BmpE_t ** pBmpMaxRE);
void InitCloseParenMatchLoToHi(BmpParenMap_t ** ppBmpParenMap, BmpE_t ** pBmpMaxRE);
void cpByteToSubBlock(SubBlock_t *ps, const UBYTE * pb, const UINT nBits);
void cpSubBlockToByte(UBYTE * pb, const SubBlock_t *ps, const UINT nBits);
void cpByteToSubBlock(SubBlock_t *ps, MyVector<UBYTE> * pb, const UINT pos, const UINT nBits);
int compare( const void *pu1, const void *pu2 );
UINT getbit( UBYTE * pbv, UINT address );
void setbit( UBYTE * pbv, BYTE bit, UINT address );
UINT * CompressIdx( UINT * uiAuncompress, UINT n );
UINT UNCOMPRESSIDXf(UINT * uiAcompress, USHORT * usAcompress, UINT idx );
 

template <class Ttype> void dumpbinary( Ttype * data, UINT nbits, UINT fNewLine=1)
{
	UINT n = nbits/(sizeof(Ttype)*8);

	for ( UINT i=0; i<n; i++ )
	{
		Ttype mask=1<<(sizeof(Ttype)*8 - 1);
		for ( UINT j=0; j<sizeof(Ttype)*8; j++ )
		{
			if ( (mask & data[i])==0 ) std::cout << "( ";
			else                       std::cout << ") ";

			mask = mask >> 1;

		}
		if ( fNewLine )	std::cout<<std::endl;
	}
}


#endif  // __UTILS_H

